@@ -113,6 +113,7 @@ def get_genre(frame):
 
 def set_genre(frame, value):
     frame.genres = value
+    frame.encoding = encoding
     return True
 
 def genre_handler(frames):
@@ -657,8 +658,8 @@ def pic_to_bin(image):
     description = image.get(util.DESCRIPTION)
     if not description:
         description = u''
-    mime = image.get(util.MIMETYPE)
     imagetype = image.get(util.IMAGETYPE, encoding)
+    mime = image.get(util.MIMETYPE)
     if not mime:
         mime = get_mime(data)
     return APIC(encoding, mime, imagetype, description, data)
@@ -697,21 +698,21 @@ class Tag(TagBase):
     revmapping = {}
 
     def __init__(self, filename=None):
-            self.__images = []
+        self.__images = []
 
-            self.__tags = CaselessDict() #Used as storage.
-            #Each key as the is the field as used by puddletag, eg. 'artist'
-            #Each value contains an mutagen.id3.Frame object
-            #that have two methods, get_value and set_value.
-            #get_value returns the value stored by the frame as
-            #text/unicode list.
-            #set_value should take text/unicode list in parse it into
-            #it understands.
+        self.__tags = CaselessDict() #Used as storage.
+        #Each key as the is the field as used by puddletag, eg. 'artist'
+        #Each value contains an mutagen.id3.Frame object
+        #that have two methods, get_value and set_value.
+        #get_value returns the value stored by the frame as
+        #text/unicode list.
+        #set_value should take text/unicode list in parse it into
+        #it understands.
 
-            #When saving the frame stored will be used. If it has a 'frames'
-            #attributes, those frames will be used instead.
+        #When saving the frame stored will be used. If it has a 'frames'
+        #attributes, those frames will be used instead.
 
-            util.MockTag.__init__(self, filename)
+        util.MockTag.__init__(self, filename)
 
     def get_filepath(self):
         return util.MockTag.get_filepath(self)
@@ -758,8 +759,18 @@ class Tag(TagBase):
     info = property(_info)
 
     def __contains__(self, key):
+        if key == '__image':
+            return bool(self.images)
+
+        elif key == '__total':
+            try:
+                return bool(get_total(self))
+            except (KeyError, ValueError):
+                return False
+
         if self.revmapping:
             key = self.revmapping.get(key, key)
+        
         return key in self.__tags
 
     def __deepcopy__(self, memo=None):
@@ -790,6 +801,7 @@ class Tag(TagBase):
                 tags[key] = deepcopy(value)
         cls.set_fundamentals(tags,
             self.mut_obj, deepcopy(self.images))
+        cls.filepath = self.filepath
         return cls
 
     @del_deco
@@ -931,7 +943,7 @@ class Tag(TagBase):
         old_apics = [z for z in audio if z.startswith(u'APIC')]
         if self.__images:
             newimages = []
-            for image in map(pic_to_bin, self.__images):
+            for image in filter(None, map(pic_to_bin, self.__images)):
                 i = 0
                 while image.HashKey in newimages:
                     i += 1
@@ -963,8 +975,6 @@ class Tag(TagBase):
             c.update_to_v23()
             c.save(v1=v1, v2=3)
 
-        self.update_tag_list()
-
         self.__tags['__tag_read'] = u'ID3v2.4' if v2 == 4 else u'ID3v2.3'
         self.update_tag_list()
         self._originaltags = audio.keys()
