@@ -1,6 +1,8 @@
 ## -*- coding: utf-8 -*-
 
-import mutagen, time, pdb, calendar, os, logging, sys, imghdr
+import base64, calendar, imghdr, json, logging, pdb, os, sys, time
+import mutagen
+
 from errno import ENOENT
 from decimal import Decimal
 from copy import copy, deepcopy
@@ -29,6 +31,11 @@ fn_hash = {
 
 splitext = lambda x: path.splitext(x)[1][1:].lower()
 
+def b64_to_img(img):
+    ret = img.copy()
+    ret['data'] = base64.b64decode(img['data'])
+    return ret
+
 def commonimages(images):
     """Checks the equality of the list of images.
 
@@ -71,6 +78,7 @@ def commontags(audios):
     tags = {}
     images = []
     imagetags = set()
+
     for audio in audios:
         if getattr(audio, 'IMAGETAGS', []):
             image = audio[IMAGE_FIELD] if audio[IMAGE_FIELD] else []
@@ -78,16 +86,21 @@ def commontags(audios):
         else:
             image = []
         images.append(image)
+        
         if hasattr(audio, 'usertags'):
             audio = audio.usertags
         else:
             audio = usertags(audio)
 
         for field, value in audio.items():
+            value = list(value) if not isinstance(value, basestring) else [value]
             if field in combined:
                 if combined[field] == value:
                     tags[field] += 1
-            else:
+                else:
+                    values = combined[field]
+                    [values.append(v) for v in value if v not in values]
+            else:               
                 combined[field] = value
                 tags[field] = 1
     combined['__image'] = commonimages(images)
@@ -122,7 +135,10 @@ def cover_info(images, d=None):
             info[IMAGE_MIMETYPE] = get_mime(image[DATA])
 
         if IMAGETYPE in image:
-            info[IMAGE_TYPE_FIELD] = IMAGETYPES[image[IMAGETYPE]]
+            try:
+                info[IMAGE_TYPE_FIELD] = IMAGETYPES[image[IMAGETYPE]]
+            except IndexError:
+                info[IMAGE_TYPE_FIELD] = IMAGETYPES[DEFAULT_COVER]
 
     if d:
         if not info[IMAGE_MIMETYPE]:
@@ -169,8 +185,6 @@ def getdeco(func):
         mapping = self.revmapping
         if key in mapping:
             return func(self, mapping[key])
-        elif key in self.mapping:
-            raise KeyError(key)
         return func(self, key)
     return f
 
@@ -238,6 +252,11 @@ def get_total(tag):
     except IndexError:
         raise KeyError('__total')
 
+def img_to_b64(img):
+    ret = img.copy()
+    ret['data'] = base64.b64encode(img['data'])
+    return ret
+        
 def info_to_dict(info):
     """Create a dictionary representation of info's attributes.
 
@@ -285,6 +304,9 @@ def info_to_dict(info):
     try: tags['__version'] = unicode(info.version)
     except AttributeError: pass
 
+    try: tags['__md5sig'] = unicode(info.md5_signature)
+    except AttributeError: pass
+
     return tags
 
 def isempty(value):
@@ -360,7 +382,7 @@ _image_defaults = {
     DESCRIPTION: lambda i: i.get(DESCRIPTION, u''),
     MIMETYPE: lambda i: get_mime(i[DATA]) if MIMETYPE in i else \
         get_mime(i[DATA]),
-    IMAGETYPE: lambda i: i.get(IMAGETYPE, 3),
+    IMAGETYPE: lambda i: i.get(IMAGETYPE, DEFAULT_COVER),
     DATA: lambda i: i[DATA]}
 
 def parse_image(image, keys=None):
@@ -370,7 +392,7 @@ def parse_image(image, keys=None):
     return dict((k, _image_defaults[k](image)) for k in keys)
 
 def reversedict(d):
-    return dict(((v,k) for k in d))
+    return dict((v,k) for k,v in d.iteritems())
 
 def setdeco(func):
     """Decorates the __setitem__ method of a Tag object using it's mapping.
@@ -398,8 +420,13 @@ def setmodtime(fn, atime, mtime):
 def set_total(tag, value):
     """If tag['track'] is of format x/y set's y to value."""
     track = to_string(tag['track']).split(u'/', 1)
+    
+    if isempty(value) and track:
+        tag['track'] = track[0]
+        return
+
     value = to_string(value)
-    if not (value and track) or len(track) == 1:
+    if not (value and track):
         return False
     tag['track'] = track[0] + u'/' + value
     return True
@@ -428,7 +455,7 @@ def str_filesize(size):
 
 def strfrequency(value):
     """Converts the frequency Hz to a string in kHz."""
-    return unicode(value / 1000.0)[:4] + u" kHz"
+    return u"%.1f kHz" % (value / 1000.0)
 
 def stringtags(tag, leaveNone = False):
     """Converts each value in tag to a string.
@@ -449,16 +476,19 @@ def stringtags(tag, leaveNone = False):
         if i in INFOTAGS:
             newtag[i] = v
             continue
-        if isinstance(i, int) or hasattr(v, 'items'):
+        if isinstance(i, (int, long, float)) or hasattr(v, 'items'):
             continue
 
         if leaveNone and isempty(v):
             newtag[i] = u''
             continue
-        elif (not v) or (len(v) == 1 and not v[0]):
+        elif (not v) or (hasattr(v, '__iter__') and len(v) == 1 and not v[0]):
             continue
+
         if isinstance(v, basestring):
             newtag[i] = v
+        elif isinstance(v, (int, float)):
+            newtag[i] = unicode(v)
         elif isinstance(i, basestring) and not isinstance(v, basestring):
             newtag[i] = v[0]
         else:
@@ -480,6 +510,42 @@ def strtime(seconds):
     """Converts UNIX time(in seconds) to more Human Readable format."""
     return time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime(seconds))
 
+def tag_to_json(audio, fields=None):
+
+    import puddlestuff.audioinfo as audioinfo
+    if isinstance(audio, basestring):
+        try:
+            audio = audioinfo.Tag(audio)
+        except:
+            import traceback
+            traceback.print_exc()
+            print "Invalid File:", audio
+            return
+
+    if audio is None:
+        return
+    try:
+        if fields:
+            tags = dict((k, audio[k]) for k in fields if k in audio)
+        else:
+            fields = []
+            tags = audio.tags.copy()
+    except AttributeError:
+        tags = audio.copy()
+
+    try:
+        if not fields or '__image' in fields:
+            if audio.images:
+                tags['__image'] = map(img_to_b64, audio.images)
+            elif '__image' in tags:
+                tags['__image'] = map(img_to_b64, tags['__image'])
+    except AttributeError:
+        if not fields or '__image' in fields:
+            if '__image' in tags:
+                tags['__image'] = map(img_to_b64, tags['__image'])
+
+    return tags
+    
 def to_string(value, errors='strict'):
     """Convert the value to a unicode string.
 
@@ -497,8 +563,9 @@ def to_string(value, errors='strict'):
 
 def usertags(tag):
     """Return dictionary of all editable key, value pairs found in tag."""
-    return dict([(z,v) for z,v in tag.items() if
-        not (isinstance(z, (int, long)) or z.startswith('__'))])
+    ret = dict((z,v) for z,v in tag.iteritems() if isinstance(z, basestring)
+        and not z.startswith('__'))
+    return ret
 
 def unicode_list(value):
     """Modifies the passed value to a unicode list.
@@ -585,7 +652,7 @@ class MockTag(object):
     def __init__(self, filename = None):
         object.__init__(self)
         self._info = {}
-        self._filepath = ''
+        self.__filepath = u''
         if filename:
             self.link(filename)
 
