@@ -33,12 +33,19 @@ This line is further split into three parts
     The Second contains the control itself, either text, combo or check
     The third contains the default arguments as shown to the user."""
 
-from puddleobjects import (PuddleConfig, safe_name, fnmatch,
-    dircmp, natcasecmp, encode_fn, decode_fn)
-import string, pdb, sys, audioinfo, decimal, os, pyparsing, re, imp, shutil, time, unicodedata
-from operator import itemgetter
+import math
+import decimal, os, pdb, re, shutil, sys
+import string, time, traceback, unicodedata
+
 from copy import deepcopy
 from functools import partial
+from operator import itemgetter
+
+import pyparsing
+
+import puddlestuff.audioinfo as audioinfo
+from puddleobjects import (PuddleConfig, safe_name, fnmatch,
+    dircmp, natcasecmp, encode_fn, decode_fn)
 
 PATH = audioinfo.PATH
 DIRPATH = audioinfo.DIRPATH
@@ -47,8 +54,9 @@ true = u'1'
 false = u'0'
 path = os.path
 
+D = decimal.Decimal
+
 def add(text,text1):
-    D = decimal.Decimal
     try:
         return unicode((D(unicode(text)) + D(unicode(text1))).normalize().to_eng_string())
     except decimal.InvalidOperation:
@@ -94,22 +102,30 @@ def and_(text, text1):
     return unicode(check_truth(text) and check_truth(text1))
 
 def caps(text):
+    #Capitalizes the first letter of each word in string and 
+    #converts the rest to lower case.
     return titleCase(text)
 
 def caps2(text):
-    upcase = [z for z,i in enumerate(text) if i in string.uppercase]
-    text = list(titleCase(text))
-    for z in upcase:
-        text[z] = text[z].upper()
-    return "".join(text)
+    #Capitalizes the first letter of each word in string and 
+    #leaves all other characters unchanged.
+    upcase = set(i for i, char in enumerate(text) if char.upper() == char)
+    return "".join(ch.upper() if i in upcase else ch 
+        for i, ch in enumerate(text.title()))
 
 def caps3(text):
+    #Capitalizes the first letter of the string and converts 
+    #the rest to lower case.
     try:
         start = re.search("\w", text, re.U).start(0)
     except AttributeError:
         return
     return text[:start] + text[start].upper() + text[start + 1:].lower()
 
+def ceiling(n_value):
+    
+    return math.ceil(n_value)
+
 def char(text):
     try:
         return unicode(ord(text))
@@ -121,12 +137,21 @@ def changeartist(artist, *files):
         audio['artist'] = artist
         audio.save()
 
-def div(text,text1):
-    D = decimal.Decimal
+def div(n_numerator, n_divisor):
+    if n_divisor == 0:
+        raise FuncError("Cannot divide by zero.")
     try:
-        return unicode((D(text) / D(text1)).normalize())
+        return unicode((D(n_numerator) / D(n_divisor)).normalize())
     except decimal.InvalidOperation:
         return
+    #ret = unicode(float(n_numerator) / n_divisor)
+    #if len(ret) < len(normalized):
+        #return ret
+    #else:
+        #return normalized
+
+def eql(text, text1):
+    return true if text == text1 else false
 
 # Contributed by Stjujsckij Nickolaj
 def enconvert(text, enc_name):
@@ -134,12 +159,12 @@ def enconvert(text, enc_name):
 &Encoding, combo, cp1250, cp1251, cp1252, cp1253, cp1254, cp1255, cp1256, cp1257, cp1258'''
     return text.encode("latin1", 'replace').decode(enc_name, 'replace')
 
-def filenametotag(m_tags, pattern):
+def filenametotag(m_tags, p_pattern):
     """Filename to Tag, File->Tag '$1'
 &Pattern, text"""
-    return findfunc.filenametotag(pattern, m_tags[PATH], True)
+    return findfunc.filenametotag(p_pattern, m_tags[PATH], True)
 
-def finddups(tracks, key = 'title', method=None):
+def finddups(tracks, key='title', method=None):
     from puddleobjects import dupes
     li = []
     for z in tracks:
@@ -149,10 +174,13 @@ def finddups(tracks, key = 'title', method=None):
             li.append(None)
     return dupes(li, method)
 
-def formatValue(m_tags, pattern, state=None):
+def floor(n_value):
+    return math.floor(n_value)
+
+def formatValue(m_tags, p_pattern, state=None):
     """Format value, Format $0 using $1
 &Format string, text"""
-    ret = findfunc.tagtofilename(pattern, m_tags, state=state)
+    ret = findfunc.parsefunc(p_pattern, m_tags, state=state)
     if not ret:
         return
     else:
@@ -161,19 +189,40 @@ def formatValue(m_tags, pattern, state=None):
 format_value = formatValue
 
 def geql(text,text1):
+
+    try:
+        text = float(text)
+    except (TypeError, ValueError):
+        pass
+
+    try:
+        text1 = float(text1)
+    except (TypeError, ValueError):
+        pass
+    
     if text >= text1:
         return true
     else:
         return false
 
 def grtr(text, text1):
+    try:
+        text = float(text) if text.strip() else 0
+    except (TypeError, ValueError):
+        pass
+
+    try:
+        text1 = float(text1) if text1.strip() else 0
+    except (TypeError, ValueError):
+        pass
+
     if text > text1:
         return true
     else:
         return false
 
-def hasformat(pattern, tagname = "__filename"):
-    if findfunc.filenametotag(pattern, tagname):
+def hasformat(p_pat, tagname = "__filename"):
+    if findfunc.filenametotag(p_pat, tagname):
         return true
     return false
 
@@ -192,12 +241,12 @@ def iflonger(a, b, text, text1):
     except TypeError:
         return
 
-def import_text(m_tags, pattern, r_tags):
+def import_text(m_tags, p_pattern, r_tags):
     '''Import text file, "Text File: $0, '$1'"
 &Pattern (can be relative path), text, lyrics.txt'''
     path = os.path
     dirpath = r_tags.dirpath
-    filename = tag_to_filename(pattern, m_tags, r_tags, False)
+    filename = tag_to_filename(p_pattern, m_tags, r_tags, False)
     if not filename:
         return
     try:
@@ -223,12 +272,34 @@ def len_(text):
     return unicode(len(text))
 
 def leql(text,text1):
+
+    try:
+        text = float(text)
+    except (TypeError, ValueError):
+        pass
+
+    try:
+        text1 = float(text1)
+    except (TypeError, ValueError):
+        pass
+    
     if text <= text1:
         return true
     else:
         return false
 
 def less(text, text1):
+
+    try:
+        text = float(text)
+    except (TypeError, ValueError):
+        pass
+
+    try:
+        text1 = float(text1)
+    except (TypeError, ValueError):
+        pass
+    
     if text < text1:
         return true
     else:
@@ -255,39 +326,27 @@ def load_images(r_tags, filepatterns, desc, matchcase,state=None):
 &Default description (can be pattern):, text
 Match filename's &case:, check'''
     tags = r_tags
-    if state is None:
-        state = {}
-    
-    dirpath = tags.dirpath
-    key = 'image_dirpath' + dirpath
-    if key in state:
-        files = state[key]
-    else:
-        files = os.listdir(dirpath)
+
     images = []
-    pictures = fnmatch(filepatterns, files, matchcase)
+    dirpath = r_tags.dirpath
+    pictures = fnmatch(filepatterns, os.listdir(dirpath), matchcase)
     for pic in pictures:
-        filename = os.path.join(dirpath, pic)
-        key = 'loaded_image' + filename
-        if key in state:
-            image = deepcopy(state[key])
-        else:
-            image = _load_image(filename)
-            if not image:
-                continue
-            desc = formatValue(tags, desc)
-            if desc is None:
-                desc = u''
-            image[audioinfo.DESCRIPTION] = desc
-            image[audioinfo.IMAGETYPE] = 3
-            state[key] = image
+        filename = os.path.join(dirpath, pic)        
+        image = _load_image(filename)
+        if not image:
+            continue
+        desc = formatValue(tags, desc)
+        if desc is None:
+            desc = u''
+        image[audioinfo.DESCRIPTION] = desc
+        image[audioinfo.IMAGETYPE] = 3
         images.append(image)
 
     if images:
         return {'__image': images}
 
 def lower(text):
-    return string.lower(text)
+    return text.lower()
 
 def merge_values(m_text, separator=u';'):
     '''Merge field, "Merge field: $0, sep='$1'"
@@ -297,20 +356,21 @@ def merge_values(m_text, separator=u';'):
     else:
         return separator.join(m_text)
 
-def meta_sep(m_tags, field, sep=u', '):
-    value = m_tags.get(field)
+def meta_sep(m_tags, p_field, p_sep=u', '):
+    value = m_tags.get(p_field)
     if value is None:
         return None
     elif isinstance(value, basestring):
         return value
     else:
-        return sep.join(value)
+        return p_sep.join(value)
 
 def meta(m_tags, field, n_index=None):
     value = m_tags.get(field)
     if value is None:
         return None
     if n_index is not None:
+        n_index = int(n_index)
         try:
             return value[n_index]
         except IndexError:
@@ -320,18 +380,23 @@ def meta(m_tags, field, n_index=None):
             return value
         return u', '.join(value)
 
-def mid(text, n, i):
+def mid(text, n_start, n_len):
+    
     try:
-        n = int(n)
-        i = int(i)
-        return unicode(text)[n: n + i]
+        n_start = int(n_start)
     except (TypeError, ValueError):
-        raise FuncError(u'Integer expected, got "%s"' % unicode(n))
-
-def mod(text,text1):
-    D = decimal.Decimal
+        raise FuncError(u'Integer expected, got "%s"' % unicode(n_start))
+    
     try:
-        return unicode((D(text) % D(text1).normalize()))
+        n_len = int(n_len)
+    except (TypeError, ValueError):
+        raise FuncError(u'Integer expected, got "%s"' % unicode(n_len))
+
+    return unicode(text)[n_start: n_start + n_len]
+
+def mod(n_x, n_y):
+    try:
+        return unicode((n_x % n_y).normalize())
     except decimal.InvalidOperation:
         return
 
@@ -344,42 +409,45 @@ def tag_to_filename(pattern, m_tags, r_tags, ext=True, state=None,
         state = {}
 
     tags = m_tags
-    tf = findfunc.tagtofilename
+    tf = findfunc.parsefunc
     path_join = os.path.join
 
     if state is None:
         state = {}
 
-    if os.path.isabs(pattern):
+    path_seps, text = tf(pattern, tags, state=state, path_sep=u"/")
 
-        new_name = lambda d: safe_name(tf(d, tags, state=state))
-        subdirs = pattern.split(u'/')
-        newdirs = map(new_name, subdirs[1:-1])
-        newdirs.append(safe_name(tf(subdirs[-1], tags, ext, state=state)))
-        newdirs.insert(0, u'/')
-        return encode_fn(path_join(*newdirs))
+    start_pos = 0
+    new_dirs = []
+    
+    if path_seps:
+        for p in path_seps:
+            if start_pos != p:
+                new_dirs.append(safe_name(text[start_pos:p]))
+            start_pos = p
+        new_dirs.append(safe_name(text[start_pos + 1:]))
     else:
-        new_name = lambda d: encode_fn(safe_name(tf(d, tags, state=state)))
-        subdirs = pattern.split(u'/')
-        count = pattern.count(u'/')
+        new_dirs = [safe_name(text)]
 
-        newdirs = map(new_name, subdirs[:-1])
-        newdirs.append(
-            encode_fn(safe_name(tf(subdirs[-1], tags, ext, state=state))))
+    if os.path.isabs(pattern):
+        return add_extension(u'/' + u'/'.join(map(safe_name, new_dirs)), tags, ext)
+    else:
 
+        subdirs = new_dirs
+        count = len(path_seps)
         
         dirpath = r_tags.dirpath
 
-        new_fn = encode_fn(path_join(*newdirs))
+        new_fn = encode_fn(path_join(*new_dirs))
         
         if new_fn.startswith('./'):
-            return path_join(dirpath, new_fn[len('./'):])
+            return add_extension(path_join(dirpath, new_fn[len('./'):]), tags, ext)
         elif new_fn.startswith('../'):
             parent = dirpath
             while new_fn.startswith('../'):
                 parent = os.path.dirname(parent)
                 new_fn = new_fn[len('../'):]
-            return path_join(parent, new_fn)
+            return add_extension(path_join(parent, new_fn), tags, ext)
         elif count and u'..' not in subdirs:
             subdirs = dirpath.split('/')
             if count >= len(subdirs):
@@ -396,9 +464,17 @@ def tag_to_filename(pattern, m_tags, r_tags, ext=True, state=None,
 
         if not parent[0]:
             parent.insert(0, '/')
-        return os.path.join(*(parent + [new_fn]))
+        return add_extension(os.path.join(*(parent + [new_fn])), tags, ext)
 
-def move(m_tags, pattern, r_tags, ext=True, state=None):
+def add_extension(fn, tags, addext = None, extension=None):
+    if not addext:
+        return fn
+    elif addext and (extension is not None):
+        return fn + os.path.extsep + encode_fn(extension)
+    else:
+        return fn + os.path.extsep + encode_fn(tags["__ext"])
+
+def move(m_tags, p_pattern, r_tags, ext=True, state=None):
     """Tag to filename, Tag->File: $1
 &Pattern, text"""
 
@@ -406,15 +482,15 @@ def move(m_tags, pattern, r_tags, ext=True, state=None):
     tags = m_tags
     tf = findfunc.tagtofilename
 
-    fn = tag_to_filename(pattern, m_tags, r_tags, ext, state)
+    fn = tag_to_filename(p_pattern, m_tags, r_tags, ext, state)
 
     if fn:
         return {'__path': fn}
 
-def mul(text, text1):
+def mul(n_x, n_y):
     D = decimal.Decimal
     try:
-        return unicode((D(text) * D(text1)).normalize())
+        return unicode((D(n_x) * D(n_y)).normalize())
     except decimal.InvalidOperation:
         return
 
@@ -425,49 +501,52 @@ def neql(text, text1):
         return true
 
 def not_(text):
-    if check_truth(text):
-        return false
-    return true
+    return false if check_truth(text) else true
 
-def num(text, numlen):
+def num(text, n_len, add_sep=false):
     if not text:
         return u""
-    text=str(text)
-    index = text.find("/")
-    if index >= 0:
-        text = text[:index]
-    try:
-        numlen = long(numlen)
-    except ValueError:
-        return text
-    if len(text) < numlen:
-        text='0' * (numlen - len(text)) + text
-    if len(text)>numlen:
-        while text.startswith('0') and len(text)>numlen:
-            text=text[1:]
-    return text
+    n_len = int(n_len)
 
-def odd(text):
-    D = decimal.Decimal
-    if D(text) % 2 != 0:
-        return true
+    sep_index = text.find(u"/")
+
+    if sep_index >= 0:
+        total = text[sep_index + 1:]
+        tracknum = text[:sep_index]
     else:
-        return false
+        total = None
+        tracknum = text
+
+    tracknum = tracknum.lstrip(u'0')
+
+    if total and check_truth(add_sep):
+        return u"%s/%s" % (tracknum.zfill(n_len), total)
+    else:
+        return tracknum.zfill(n_len)
+
+def odd(n_number):
+    return true if (n_number % 2 != 0) else false
 
 def or_(text,text1):
-    return unicode(check_truth(text) or check_truth(text1))
+    return true if (check_truth(text) or check_truth(text1)) else false
 
 def rand():
     import random
     return unicode(random.random())
 
-def re_escape(rex):
+def _round(n_value):
+    return round(n_value)
+
+def re_escape(rex, chars=r'^$[]\+*?.(){},|'):
     escaped = u""
     for ch in rex:
-        if ch in r'^$[]\+*?.(){},|' : escaped = escaped + u'\\' + ch
+        if ch in chars : escaped = escaped + u'\\' + ch
         else: escaped = escaped + ch
     return escaped
 
+def pat_escape(p_pat):
+    return re_escape(p_pat, u'$%\\')
+
 def remove_fields():
     '''Remove Fields, <blank> $0'''
     return u''
@@ -553,79 +632,68 @@ only as &whole word, check'''
             raise findfunc.FuncError(unicode(e))
     return text
 
-def replacetokens(text, dictionary, default=u''):
+class RegHelper(object):
+    def __init__(self, groups):
+        self.groups = groups
 
-    pat = re.compile('\$\d+')
-    start = 0
-    match = pat.search(text, start)
-    if match:
-        start = match.start()
-        l = [text[:start]]
-        end = start
-    else:
-        return default
-    while match:
-        start = match.start()
-        l.append(text[end:start])
-        end = match.end()
-        sub = int(text[start+1: end])
+    def repl(self, match):
+        v = int(match.group()[1:])
         try:
-            l.append(dictionary[sub])
+            return re_escape(self.groups[v], u'"\\,')
         except KeyError:
-            l.append(text[start: end])
-        match = pat.search(text, end)
-    l.append(text[end:])
-    return u''.join(l)
+            return u'""'
 
-def replaceWithReg(text, expr, rep, matchcase=False):
+def replaceWithReg(m_tags, text, regex, repl, matchcase=False, state=None):
     """Replace with RegExp, "RegReplace $0: RegExp '$1' with '$2', Match Case: $3"
 &Regular Expression, text
 Replace &matches with:, text
 Match &Case, check"""
-    if not expr:
+
+    if not regex:
         return text
+
+    if not check_truth(matchcase):
+        flags = re.UNICODE | re.I
+    else:
+        flags = re.UNICODE
+
+    def replace_tokens(match):
+        groups = match.groups()
+        group = match.group()
+        if not group:
+            d = {}
+        elif groups:
+            d = dict((i + 1, g) for i, g in enumerate(groups))
+            d[0] = group
+        else:
+            d = {1: group, 0: group}
+
+
+        ret = re.sub('(?i)\$\d+', RegHelper(d).repl, repl, 0)
+        
+        return findfunc.parsefunc(ret, m_tags)
+
     try:
-        match = re.search(expr, text, re.I | re.UNICODE)
+        try:
+            return re.sub(regex, replace_tokens, text, 0, flags)
+        except TypeError:
+            #Python2.6 doesn't accept flags arg.
+            if matchcase:
+                return re.sub(u'(?i)' + regex, replace_tokens, text, 0)
+            else:
+                return re.sub(regex, replace_tokens, text, 0)
     except re.error, e:
         raise findfunc.FuncError(unicode(e))
 
-    ret = []
-    unmatched = 0
-    append = ret.append
-    
-    if not matchcase:
-        matches = re.finditer(expr, text, re.I | re.UNICODE)
-    else:
-        matches = re.finditer(expr, text, re.UNICODE)
-
-    for match in matches:
-        group = match.group()
-        groups = match.groups()
-        if not group:
-            continue
-        if groups:
-            d = dict((i + 1, g) for i, g in enumerate(groups))
-        else:
-            d = {1: group}
-
-        replacetext = findfunc.parsefunc(rep, {}, d)
-        replacetext = replacetokens(replacetext, d, replacetext)
-
-        append(text[unmatched:match.start(0)])
-        unmatched = match.end(0)
-        append(replacetext)
-    ret.append(text[unmatched:])
-    return u''.join(ret)
-
 replace_regex = replaceWithReg
 
-validFilenameChars = "'-_.!()[]{}&~+^ %s%s%s" % (
+VALID_FILENAME_CHARS = "'-_.!()[]{}&~+^ %s%s%s" % (
     string.ascii_letters, string.digits, os.path.sep)
 
 #Contributed by Erik Reckase
-def removeDisallowedFilenameChars(t_filename):
-    cleanedFilename = unicodedata.normalize('NFKD', t_filename).encode('ASCII', 'ignore')
-    return u''.join(c for c in cleanedFilename if c in validFilenameChars)
+def to_ascii(t_fn):
+    cleaned_fn = unicodedata.normalize('NFKD', t_fn).encode('ASCII', 'ignore')
+    return u''.join(c for c in cleaned_fn if c in VALID_FILENAME_CHARS)
 
 def remove_dupes(m_text, matchcase=False):
     """Remove duplicate values, "Remove Dupes: $0, Match Case $1"
@@ -634,9 +702,10 @@ Match &Case, check"""
     if isinstance(text, basestring):
         return text
 
+    
     if matchcase:
         ret = []
-        append = temp.append
+        append = ret.append
         [append(z) for z in text if z not in ret]
         return ret
     else:
@@ -657,6 +726,45 @@ def right(text,n):
         return u''
     return text[-int(n):]
 
+def gain_to_watts(gain):
+    return 10**(-gain*.1)
+
+def to_hexstring(x):
+    # leading space required; blame Apple
+    return " %08X" % int(x)
+
+def rg2sc(gain, peak=None):
+    if peak is None:
+        gain = gain.split(u':')
+        if len(gain) == 2: #gain:peak
+            peak = float(gain[1])
+            gain = float(gain[0])
+            print peak, gain
+        elif len(gain) == 3: #channel:gain:peak
+            peak = float(gain[2])
+            gain = float(gain[1])
+            print gain, peak
+        else:
+            return
+    else:
+        gain = float(gain)
+        peak = float(peak)
+
+    values = [
+        to_hexstring(1000 * gain_to_watts(gain)),
+        to_hexstring(1000 * gain_to_watts(gain)),
+        to_hexstring(2500 * gain_to_watts(gain)),
+        to_hexstring(2500 * gain_to_watts(gain)),
+        " 00024CA8", # bogus
+        " 00024CA8", # bogus
+        to_hexstring(peak * (32*1024 - 1)),
+        to_hexstring(peak * (32*1024 - 1)),
+        " 00024CA8", # bogus
+        " 00024CA8", # bogus
+        ]
+
+    return unicode(''.join(values))
+
 def save_artwork(m_tags, pattern, r_tags, state=None, write=True):
     """Export artwork to file, "Export Art: pattern='$1'"
 &Pattern (extension not required), text, folder_%img_counter%"""
@@ -751,12 +859,8 @@ def find(text, text1):
         return unicode(val)
     return u'-1'
 
-def sub(text,text1):
-    D = decimal.Decimal
-    try:
-        return  unicode(D(text) - D(text1))
-    except decimal.InvalidOperation:
-        return
+def sub(n_1, n_2):
+    return unicode(n_1 - n_2)
 
 def tag_dir(m_tags, pattern, r_tags, state = None):
     '''Tag to Dir, "Tag->Dir: $1"
@@ -786,19 +890,20 @@ def testfunction(tags, t_text, p_pattern, n_number):
     assert n_number == 23
     return u'Passed'
 
-def texttotag(tags, text, text1, text2):
+def texttotag(tags, input_text, p_pattern, output, state=None):
     """Text to Tag, "Text to Tag: $0 -> $1, $2"
 &Text, text
 &Pattern, text
 &Output, text"""
-    pattern = text1
-    tagpattern = pyparsing.Literal('%').suppress() + pyparsing.Word(pyparsing.nums)
-    d = findfunc.tagtotag(pattern, findfunc.tagtofilename(text, tags), tagpattern)
+    tagpattern = pyparsing.Literal('%').suppress() + \
+        pyparsing.Word(pyparsing.nums)
+    input_text = findfunc.parsefunc(input_text, tags, state=state)
+    d = findfunc.tagtotag(p_pattern, input_text, tagpattern)
     if d:
-        output = text2
         for key in d:
-            output = output.replace(u'%' + unicode(key), unicode(d[key]))
-        return findfunc.tagtofilename(output, tags)
+            output = output.replace(u'%' +
+                unicode(key), pat_escape(unicode(d[key])))
+        return findfunc.parsefunc(output, tags, state=state)
     return None
 
 def titleCase(text, ctype = None, characters = None):
@@ -867,11 +972,14 @@ functions = {
     "caps": caps,
     "caps2": caps2,
     "caps3": caps3,
+    "ceiling": ceiling,
     "char": char,
     "div": div,
     "enconvert": enconvert,
+    "equals": eql,
     'filenametotag': filenametotag,
     "find": find,
+    "floor": floor,
     "format": formatValue,
     "geql": geql,
     "grtr": grtr,
@@ -904,8 +1012,9 @@ functions = {
     'remove_except': remove_except,
     #'remove_tag': remove_tag,
     "replace": replace,
-    "replaceWithReg": replaceWithReg,
+    "regex": replaceWithReg,
     "right": right,
+    "round": _round,
     'save_artwork': save_artwork,
     'sort': sort_field,
     'split_by_sep': split_by_sep,
@@ -919,7 +1028,7 @@ functions = {
     "upper": upper,
     'update_from_tag': update_from_tag,
     "validate": validate,
-    'to_ascii': removeDisallowedFilenameChars}
+    'to_ascii': to_ascii}
 
 no_fields = (filenametotag, load_images, move, remove_except,
     save_artwork, tag_dir, update_from_tag)
