@@ -10,16 +10,18 @@ from functools import partial
 from PyQt4.QtGui import *
 from PyQt4.QtCore import *
 
-from puddlestuff.actiondlg import pprint_tag
+from puddlestuff.util import pprint_tag
 import puddlestuff.findfunc as findfunc
 import puddlestuff.resource
 
 from audioinfo import commontags, PATH
 from puddlestuff.constants import HOMEDIR, KEEP
-from puddleobjects import (get_icon, gettaglist, partial,
+from puddleobjects import (
+    get_icon, gettaglist, partial,
     settaglist, winsettings, ListButtons, MoveButtons, OKCancel,
-    PicWidget, PuddleConfig)
+    PicWidget, PuddleConfig, natcasecmp)
 from puddlestuff.translations import translate
+from puddlestuff.util import to_string
 
 ADD, EDIT, REMOVE = (1, 2, 3)
 UNCHANGED = 0
@@ -73,6 +75,8 @@ class AutonumberDialog(QDialog):
             self._numtracks.setValue(numtracks)
         self._restart_numbering = QCheckBox(translate('Autonumbering Wizard',
             "&Restart numbering at each directory."))
+        self.connect(self._restart_numbering, SIGNAL("stateChanged(int)"),
+            self.enableNumTracks)
 
         vbox.addLayout(hbox(self._separator, self._numtracks))
         vbox.addWidget(self._restart_numbering)
@@ -90,17 +94,33 @@ class AutonumberDialog(QDialog):
 
         self._loadSettings()
 
-    def emitValuesAndSave(self):
-        self.close()
-        if self._separator.checkState() == Qt.Checked:
-            self.emit(SIGNAL("newtracks"), self._start.value(),
-                        self._numtracks.value(),
-                        self._restart_numbering.checkState(),
-                        self._padlength.value())
+    def enableNumTracks(self, state):
+        if state == Qt.Checked:
+            self._numtracks.setVisible(False)
+            self._separator.setText(translate('Autonumbering Wizard',
+            "Add track &separator ['/']"))
         else:
-            self.emit(SIGNAL("newtracks"), self._start.value(),
-                        None, self._restart_numbering.checkState(),
-                        self._padlength.value())
+            self._numtracks.setVisible(True)
+            self._separator.setText(translate('Autonumbering Wizard',
+            "Add track &separator ['/']: Number of tracks"))
+
+    def emitValuesAndSave(self):
+        if self._numtracks.isVisible():
+            if self._separator.isChecked():
+                numtracks = self._numtracks.value()
+            else:
+                numtracks = -1 #Don't use totals
+        else:
+            if self._separator.isChecked():
+                numtracks = -2 #Use totals, automaticall generated
+            else:
+                numtracks = -1 
+
+        self.close()
+        
+        self.emit(SIGNAL("newtracks"), self._start.value(),
+            numtracks, self._restart_numbering.checkState(),
+            self._padlength.value())
         self._saveSettings()
 
     def _loadSettings(self):
@@ -305,6 +325,7 @@ class EditField(QDialog):
         label = QLabel(translate('Edit Field', "&Field"))
         self.tagcombo = QComboBox()
         self.tagcombo.setEditable(True)
+        label.setBuddy(self.tagcombo)
         completer = self.tagcombo.completer()
         completer.setCaseSensitivity(Qt.CaseSensitive)
         completer.setCompletionMode(QCompleter.UnfilteredPopupCompletion)
@@ -419,6 +440,88 @@ class VerticalHeader(QHeaderView):
     def _resize(self, row, oldsize, newsize):
         self.setDefaultSectionSize(newsize)
 
+class StatusWidgetCombo(QComboBox):
+    def __init__(self, items=None, status=None, colors=None, preview=False):
+        QComboBox.__init__(self)
+        self.preview = preview
+
+        self.statusColors = colors
+        self.setSizeAdjustPolicy(QComboBox.AdjustToMinimumContentsLength)
+        
+        items = sorted(items, cmp=natcasecmp)
+        if len(items) > 1:
+            items.append(r'\\'.join(items))
+        self.addItem(KEEP)
+        self.addItems(items)
+        self.setCurrentIndex(0)
+            
+        if status and status in self.statusColors:
+            self.setBackground(self.statusColors[status])
+        else:
+            self.setBackground(None)
+
+        self._status = status 
+        self._original = (items, preview, status)
+        self.linked = []
+
+    def _get_preview(self):
+        return False
+        return self.font().bold()
+
+    def _set_preview(self, value):
+        return
+        font = self.font()
+        font.setBold(value)
+        self.setFont(font)
+
+    preview = property(_get_preview, _set_preview)
+
+    def _get_status(self):
+        return self._status
+
+    def _set_status(self, status):
+        if status and status in self.statusColors:
+            if self._status == ADD and status != REMOVE:
+                return
+            self.setBackground(self.statusColors[status])
+            self._status = status
+        else:
+            self.setBackground(None)
+            self._status = None
+
+    status = property(_get_status, _set_status)
+
+    def setBackground(self, brush = None):
+        if brush is None:
+            color = QLineEdit().palette().color(QPalette.Base).name()
+        else:
+            color = brush.color().name()
+        self.setStyleSheet("QComboBox { background-color: %s; }" % color);
+
+    def background(self):
+        brush = QBrush()
+        brush.setColor(self.palette().color(QPalette.Background))
+        return brush
+
+    def reset(self):
+        self.clear()
+        self.addItems(self._original[0])
+        self.setCurrentIndex(0)
+        self.preview = self._original[1]
+        status = self._original[2]
+        if status == ADD:
+            self.status = REMOVE
+        else:
+            self.status = status
+        self.linked = []
+
+    def setText(self, value):
+        if self.currentIndex() == 0:
+            self.insertItem(1, value)
+            self.setCurrentIndex(1)
+        else:
+            self.setItemText(self.currentIndex(), value)
+
 class ExTags(QDialog):
     """A dialog that shows you the tags in a file
 
@@ -561,7 +664,7 @@ class ExTags(QDialog):
         else:
             self.table.setEnabled(True)
             self._reset.setEnabled(True)
-            if len(self.table.selectedItems()) / 2 > 1:
+            if len(self.table.selectedIndexes()) / 2 > 1:
                 self.listbuttons.edit.setEnabled(False)
                 self.listbuttons.duplicate.setEnabled(False)
             else:
@@ -625,7 +728,7 @@ class ExTags(QDialog):
                     if row + 1 < rowcount:
                         self.table.selectRow(row + 1)
                 else:
-                    cur_item = self.table.currentItem()
+                    cur_item = self.table.item(self.table.currentRow(),0)
                     self.resetFields([cur_item])
                     self.table.setCurrentItem(cur_item,
                         QItemSelectionModel.ClearAndSelect)
@@ -644,7 +747,10 @@ class ExTags(QDialog):
         getitem = self.table.item
         item = getitem(row, 0)
         tag = unicode(item.text())
-        value = unicode(getitem(row, 1).text())
+        try:
+            value = unicode(getitem(row, 1).text())
+        except AttributeError:
+            value = unicode(self.table.cellWidget(row, 1).currentText())
         if status:
             return (tag, value, item.status)
         else:
@@ -666,10 +772,13 @@ class ExTags(QDialog):
 
         item = self.table.item
         for row in xrange(self.table.rowCount()):
-            item(row, 0).statusColors = self._colors
-            item(row, 0).status = item(row, 0).status
-            item(row, 1).statusColors = self._colors
-            item(row, 1).status = item(row, 1).status
+            field_item = self.get_item(row, 0)
+            field_item.statusColors = self._colors
+            field_item.status = field_item.status
+            
+            val_item = self.get_item(row, 1)
+            val_item.statusColors = self._colors
+            val_item.status = val_item.status
 
     def listtotag(self):
         get_field = self.get_field
@@ -687,7 +796,7 @@ class ExTags(QDialog):
                     tags[lowered[l_field]].append(val)
                 else:
                     lowered[l_field] = field
-                    tags[field] = [val]
+                    tags[field] = [z.strip() for z in val.split('\\') if z.strip()]
             else:
                 if field.lower() not in lowered:
                     tags[field] = []
@@ -711,7 +820,9 @@ class ExTags(QDialog):
         else:
             self.setWindowTitle(
                 translate('Extended Tags', 'Different files.'))
-
+            
+            from puddlestuff.tagmodel import status
+            k = status['table'].model().taginfo[0]
             common, numvalues, imagetags = commontags(audios)
             images = common['__image']
             del(common['__image'])
@@ -734,9 +845,8 @@ class ExTags(QDialog):
                     preview = BOLD
                 else:
                     preview = UNCHANGED
-
                 if numvalues[field] != len(audios):
-                    self._settag(row, field, KEEP)
+                    self._settag(row, field, values, multi=True)
                     row += 1
                 else:
                     if isinstance(values, basestring):
@@ -806,18 +916,28 @@ class ExTags(QDialog):
         self.loadFiles([self._files[row]])
         self.emit(SIGNAL('rowChanged'), row)
 
+    def get_item(self, row, column=None):
+        if column is None: #Assume QModelIndex passed
+            column = row.column()
+            row = row.row()
+        item = self.table.item(row, column)
+        if item is None:
+            item = self.table.cellWidget(row, column)
+        return item
+
     def removeField(self):
         tb = self.table
         tb.setSortingEnabled(False)
         to_remove = {}
         rows = []
-        for i in self.table.selectedItems():
-            row = tb.row(i)
-            if i.status == ADD:
-                to_remove[row] = i
+        for index in self.table.selectedIndexes():
+            row = index.row()
+            item = self.get_item(index)
+            if item.status == ADD:
+                to_remove[row] = item
             rows.append(row)
-            i.status = REMOVE
-            i.status = REMOVE
+            item.status = REMOVE
+            item.status = REMOVE
         [tb.removeRow(tb.row(z)) for z in to_remove.values()]
         tb.setSortingEnabled(True)
         self.filechanged = True
@@ -832,18 +952,19 @@ class ExTags(QDialog):
         box = self.table
         to_remove = {} #Stores row: item values so that only one item
                        #gets removed per row.
-        if items is None:
-            items = box.selectedItems()
 
         max_row = -1
-        for item in box.selectedItems():
+        for index in box.selectedIndexes():
+            row = index.row()
+            item = self.table.item(row, index.column())
+            if item is None:    
+                item = self.table.cellWidget(row, index.column())
             for i in item.linked:
                 try:
                     to_remove[box.row(i)] = i
                 except RuntimeError:
                     pass
             item.reset()
-            row = self.table.row(item)
             if row > max_row:
                 max_row = row
             if item.status == REMOVE:
@@ -872,8 +993,18 @@ class ExTags(QDialog):
             self.piclabel.setImages(None)
 
     def save(self):
+        
+        if not self.filechanged:
+            table = self.table
+            for row in range(table.rowCount()):
+                combo = table.cellWidget(row, 1)
+                if combo is not None and combo.currentIndex() != 0:
+                    self.filechanged = True
+                    break
+
         if not self.filechanged:
             return
+
         tags = self.listtotag()
         if self.piclabel.context != 'Cover Varies':
             if not self.piclabel.images:
@@ -886,7 +1017,7 @@ class ExTags(QDialog):
         self.emit(SIGNAL('extendedtags'), tags)
 
     def _settag(self, row, field, value, status=None, preview=False,
-        check=False):
+                check=False, multi=False):
 
         tb = self.table
         tb.setSortingEnabled(False)
@@ -895,16 +1026,20 @@ class ExTags(QDialog):
             field_item = StatusWidgetItem(field, status,
                 self._colors, preview)
             tb.setItem(row, 0, field_item)
-            valitem = StatusWidgetItem(value, status, self._colors, preview)
-            tb.setItem(row, 1, valitem)
+            if not multi and (len(value) == 1 or isinstance(value, basestring)):
+                valitem = StatusWidgetItem(to_string(value), status, self._colors, preview)
+                tb.setItem(row, 1, valitem)
+            else:
+                valitem = StatusWidgetCombo(value, status, self._colors, preview)
+                tb.setCellWidget(row, 1, valitem)
         else:
             field_item = tb.item(row, 0)
             field_item.setText(field)
             field_item.status = status
-
-            valitem = tb.item(row, 1)
-            valitem.setText(value)
-            valitem.status = status
+            
+            val_item = self.get_item(row, 1)
+            val_item.setText(value)
+            val_item.status = status
 
         if check:
             lowered_tag = field.lower()
@@ -915,15 +1050,65 @@ class ExTags(QDialog):
                     item.setText(field)
                     if item.status not in [ADD, REMOVE]:
                         item.status = EDIT
-                        tb.item(row, 1).status = EDIT
+                        try:
+                            tb.item(row, 1).status = EDIT
+                        except AttributeError:
+                            tb.cellWidget(row, 1).status = EDIT
 
         tb.setSortingEnabled(True)
-        return valitem
-        
+        return field_item
+
+class ConfirmationErrorDialog(QDialog):
+    def __init__(self, name, parent=None):
+        super(ConfirmationErrorDialog, self).__init__(parent)
+        self.name = name
+        self.__label = QLabel()
+        icon = QLabel()
+        msgbox = QMessageBox()
+        msgbox.setIcon(QMessageBox.Warning)
+        icon.setPixmap(msgbox.iconPixmap())
+        ok = QPushButton(translate("Defaults", "OK"))
+        checkbox = QCheckBox(
+            translate("Defaults", "Never show this message again."))
+
+        self.connect(ok, SIGNAL('clicked()'), partial(self.saveState, name))
+
+        labelbox = QHBoxLayout()
+        labelbox.addWidget(icon)
+        labelbox.addWidget(self.__label)
+
+        layout = QVBoxLayout()
+        layout.addLayout(labelbox, 1)
+        layout.addWidget(checkbox, 0, Qt.AlignHCenter)
+        layout.addWidget(ok, 0, Qt.AlignHCenter)
+        layout.addStretch()
+        self.setLayout(layout)
+
+    def saveState(self, name):
+        settings = PuddleConfig()
+        settings.set("OnceOnlyErrors", name, True)
+        self.close()
+
+    def showMessage(self, message):
+        settings = PuddleConfig()
+        should_show = settings.get("OnceOnlyErrors", self.name, True)
+        self.__label.setText(message)
+        if should_show:
+            self.setModal(True)
+            self.show()
+            
 
 if __name__ == "__main__":
     app = QApplication(sys.argv)
-    wid = ImportTextFile(clipboard = True)
+    dirpath = '/home/keith/Desktop/Daft.Punk-Random.Access.Memories-2013-FLAC.-NewsHost-1023'
+    import glob
+    tags = map(puddlestuff.audioinfo.Tag, glob.glob(os.path.join(dirpath, "*.flac")))
+    for tag in tags:
+        tag.preview = {}
+        tag.equal_fields = lambda: []
+        tag.library = None
+    wid = ExTags(files=tags)
     wid.resize(200, 400)
     wid.show()
+    pyqtRemoveInputHook()
     app.exec_()
