@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+import json
 import puddlestuff.findfunc as findfunc
 from puddlestuff.puddleobjects import (dircmp, safe_name, natcasecmp,
     LongInfoMessage, PuddleConfig, PuddleDock, encode_fn, decode_fn)
@@ -7,10 +8,11 @@ from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 import os, pdb
 path = os.path
+from collections import defaultdict
 import puddlestuff.helperwin as helperwin
 from functools import partial
 from itertools import izip
-from puddlestuff.audioinfo import stringtags, PATH, DIRPATH, EXTENSION, FILETAGS
+from puddlestuff.audioinfo import stringtags, PATH, DIRPATH, EXTENSION, FILETAGS, tag_to_json
 from operator import itemgetter
 import puddlestuff.musiclib, puddlestuff.about as about
 import traceback
@@ -25,7 +27,10 @@ status = {}
 def applyaction(files=None, funcs=None):
     if files is None:
         files = status['selectedfiles']
-    r = findfunc.runAction
+    if isinstance(funcs[0], findfunc.Macro):
+        r = findfunc.apply_macros
+    else:
+        r = findfunc.apply_actions
     state = {'__total_files': unicode(len(files))}
     state['__files'] = files
     def func():
@@ -34,7 +39,11 @@ def applyaction(files=None, funcs=None):
     emit('writeaction', func(), None, state)
 
 def applyquickaction(files, funcs):
-    qa = findfunc.runQuickAction
+    if isinstance(funcs[0], findfunc.Macro):
+        qa = findfunc.apply_macros
+    else:
+        qa = findfunc.apply_actions
+    
     selected = status['selectedtags']
     state = {'__total_files': unicode(len(selected))}
     t = (qa(funcs, f, state, s.keys()) for f, s in izip(files, selected))
@@ -80,11 +89,7 @@ def connect_status(actions):
 def copy():
     selected = status['selectedtags']
     mime = QMimeData()
-    if selected[0]:
-        text = selected[0][selected[0].keys()[0]]
-        if not isinstance(text, basestring):
-            text = SEPARATOR.join(text)
-        mime.setText(text)
+    mime.setText(json.dumps(map(tag_to_json, selected)))
     ba = QByteArray(unicode(selected).encode('utf8'))
     mime.setData('application/x-puddletag-tags', ba)
     QApplication.clipboard().setMimeData(mime)
@@ -93,19 +98,24 @@ def copy_whole():
     tags = []
     mime = QMimeData()
     
-    def usertags(f):
+    def usertags(f, images=True):
         ret = f.usertags
-        if hasattr(f, 'images') and f.images:
+        if images and hasattr(f, 'images') and f.images:
             ret.update({'__image': f.images})
         return ret
         
     tags = [usertags(f) for f in status['selectedfiles']]
 
-    if tags[0]:
-        text = tags[0][tags[0].keys()[0]]
-        if not isinstance(text, basestring):
-            text = SEPARATOR.join(text)
-        mime.setText(text)
+    data = json.dumps(map(tag_to_json, tags))
+
+    to_copy = check_copy_data(data)
+    if to_copy == 2:
+        tags = [usertags(f, False) for f in status['selectedfiles']]
+        data = json.dumps(map(tag_to_json, tags))
+    elif to_copy == 1:
+        return
+
+    mime.setText(data)
     ba = QByteArray(unicode(tags).encode('utf8'))
     mime.setData('application/x-puddletag-tags', ba)
     QApplication.clipboard().setMimeData(mime)
@@ -114,12 +124,36 @@ def cut():
     selected = status['selectedtags']
     ba = QByteArray(unicode(selected).encode('utf8'))
     mime = QMimeData()
+    mime.setText(json.dumps(map(tag_to_json, selected)))
     mime.setData('application/x-puddletag-tags', ba)
     QApplication.clipboard().setMimeData(mime)
 
     emit('writeselected', (dict([(z, u"") for z in s if z not in FILETAGS])
         for s in selected))
 
+def check_copy_data(data):
+    #0 = yes
+    #1 = no
+    #2 = no images
+    print len(data)
+    if len(data) > 5242880:
+        msgbox = QMessageBox()
+        msgbox.setText(translate("Messages",
+            "That's a large amount of data to copy.\n"
+            "It may cause your system to lock up.\n\n"
+            "Do you want to go ahead?"))
+
+        msgbox.setIcon(QMessageBox.Question)
+        msgbox.addButton(translate("Defaults", "&Yes"),
+            QMessageBox.YesRole)
+        msgbox.addButton(translate("Defaults", "No"),
+            QMessageBox.NoRole)
+        msgbox.addButton(translate("Messages", "Copy without images."),
+            QMessageBox.ApplyRole)
+        return msgbox.exec_()
+    else:
+        return 0
+
 def display_tag(tag):
     """Used to display tags in the status bar in a human parseable format."""
     if not tag:
@@ -205,35 +239,40 @@ def load_musiclib(parent=None):
     obj.connect(m, SIGNAL('adddock'), emit_received('adddock'))
     m.show()
 
-_padding = u'0'
-def _pad(text, numlen):
-    if len(text) < numlen:
-        text = _padding * ((numlen - len(text)) / len(_padding)) + text
+
+def _pad(trknum, total, padlen):
+    if total is not None:
+        text = unicode(trknum).zfill(padlen) + u"/" + unicode(total)
+    else:
+        text = unicode(trknum).zfill(padlen)
     return text
 
-def number_tracks(tags, start, numtracks, restartdirs, padlength):
+def number_tracks(tags, offset, numtracks, restartdirs, padlength):
     """Numbers the selected tracks sequentially in the range
     between the indexes.
     The first item of indices is the starting track.
     The second item of indices is the number of tracks."""
-    if numtracks:
-        num = "/" + unicode(numtracks)
-    else: num = ""
 
     if restartdirs: #Restart dir numbering
-        folders = {}
-        taglist = []
-        for tag in tags:
-            folder = tag[DIRPATH]
-            if folder in folders:
-                folders[folder] += 1
-            else:
-                folders[folder] = start
-            taglist.append({"track": _pad(unicode(folders[folder]) + num,
-                padlength)})
+        folders = defaultdict(lambda: [])
+        [folders[tag.dirpath].append(i) for i, tag in enumerate(tags)]
     else:
-        taglist = [{"track": _pad(unicode(z) + num, padlength)}
-            for z in range(start, start + len(tags) + 1)]
+        folders = {'fol': [i for i, t in enumerate(tags)]}
+    
+    taglist = {}
+    for tags in folders.itervalues():
+        if numtracks == -2:
+            total = len(tags)
+        elif numtracks is -1:
+            total = None
+        elif numtracks >= 0:
+            total = numtracks
+        for trknum, index in enumerate(tags):
+            trknum += offset
+            text = _pad(trknum, total, padlength)
+            taglist[index] = {'track': text}
+
+    taglist = [v for k,v in sorted(taglist.items(), key=lambda x: x[0])]
 
     emit('writeselected', taglist)
 
@@ -303,8 +342,9 @@ def run_action(parent=None, quickaction=False):
     action_tool = PuddleDock._controls['Actions']
     parent.connect(win, SIGNAL('actionOrderChanged'),
         action_tool.updateOrder)
-    parent.connect(win, SIGNAL('checkedChanged'),
-        action_tool.updateChecked)
+    if not quickaction:
+        parent.connect(win, SIGNAL('checkedChanged'),
+            action_tool.updateChecked)
     win.show()
 
 def run_function(parent=None, prevfunc=None):
@@ -449,7 +489,7 @@ def update_status(enable = True):
             newfilename = newfilename['__path']
             emit('tfstatus', translate("Status Bar",
                 "New Filename: <b>%1</b>").arg(
-                newfilename.decode('utf8', 'replace')))
+                    decode_fn(newfilename)))
         else:
             emit('tfstatus', u'<b>No change</b>')
     except findfunc.ParseError, e:
