@@ -22,7 +22,7 @@ INDEXCHANGED = SIGNAL('currentIndexChanged(const QString&)')
 def loadsettings(filepath = None):
     settings = PuddleConfig()
     if filepath:
-        setting.filename = filepath
+        settings.filename = filepath
     else:
         settings.filename = os.path.join(settings.savedir, 'tagpanel')
     numrows = settings.get('panel','numrows',-1, True)
@@ -46,7 +46,7 @@ def loadsettings(filepath = None):
 def savesettings(d, filepath=None):
     settings = PuddleConfig()
     if filepath:
-        setting.filename = filepath
+        settings.filename = filepath
     else:
         settings.filename = os.path.join(settings.savedir, 'tagpanel')
     settings.set('panel', 'numrows', unicode(len(d)))
@@ -60,7 +60,6 @@ class Combo(QComboBox):
         super(Combo, self).__init__(*args, **kwargs)
         self._edited = False
         def k(text): self._edited = True
-        pdb.set_trace()
         self.connect(self, SIGNAL('editTextChanged(const QString&)'),
             k)        
 
@@ -269,7 +268,7 @@ class FrameCombo(QGroupBox):
         ...        ('Track', 'track'), ("Comments",'comment'), ('Genre', 'genre'), (u'Year', u'date')]
         >>>rows = {0:[0], 1:[1], 2:[2], 3[3,4,6],4:[5]
         >>>f = FrameCombo()
-        >>>f.setCombo(tags,rows)"""
+        >>>f.setCombo(tags, rows)"""
         #pdb.set_trace()
 
         if self.combos:
