@@ -14,15 +14,15 @@ from itertools import groupby # for unique function.
 from bisect import bisect_left, insort_left # for unique function.
 from copy import copy
 import audioinfo
-from audioinfo import (IMAGETYPES, DESCRIPTION, DATA, IMAGETYPE,
-    encode_fn, decode_fn)
+from audioinfo import (IMAGETYPES, DESCRIPTION, DATA, IMAGETYPE, DEFAULT_COVER,
+    encode_fn, decode_fn, INFOTAGS)
 from operator import itemgetter
 path = os.path
 from configobj import ConfigObj
 import traceback
 import time, re
 from glob import glob
-from constants import ACTIONDIR
+from constants import ACTIONDIR, SAVEDIR
 from PyQt4.QtCore import QFile, QIODevice
 from StringIO import StringIO
 import itertools
@@ -32,6 +32,19 @@ MSGARGS = (QMessageBox.Warning, QMessageBox.Yes or QMessageBox.Default,
 from functools import partial
 from puddlestuff.translations import translate
 
+# Parameters for string distance function.
+# Words that can be moved to the end of a string using a comma.
+SD_END_WORDS = ['the', 'a', 'an']
+# Reduced weights for certain portions of the string.
+SD_PATTERNS = [
+    (r'^the ', 0.1),
+    (r'[\[\(]?(ep|single)[\]\)]?', 0.0),
+    (r'[\[\(]?(featuring|feat|ft)[\. :].+', 0.1),
+    (r'\(.*?\)', 0.3),
+    (r'\[.*?\]', 0.3),
+    (r'(, )?(pt\.|part) .+', 0.2),
+]
+
 mod_keys = {
     Qt.ShiftModifier: u'Shift',
     Qt.MetaModifier: u'Meta',
@@ -173,7 +186,10 @@ class CoverButton(QPushButton):
         self.setCurrentIndex(3)
     
     def setCurrentIndex(self, index):
-        self.setText(imagetypes[index][1])
+        try:
+            self.setText(imagetypes[index][1])
+        except IndexError:
+            self.setText(imagetypes[DEFAULT_COVER][1])
         self.emit(SIGNAL('currentIndexChanged (int)'), index)
         self._index = index
     
@@ -266,11 +282,88 @@ def winsettings(name, dialog, settings):
             cevent(event)
     setattr(dialog, 'closeEvent', closeEvent)
 
-try:
-    from Levenshtein import ratio
-except ImportError:
-    from difflib import SequenceMatcher
-    ratio = lambda a,b: SequenceMatcher(None, a,b).ratio()
+#Next three functions from beets: http://code.google.com/p/beets
+
+def _levenshtein(s1, s2):
+    """A nice DP edit distance implementation from Wikibooks:
+    http://en.wikibooks.org/wiki/Algorithm_implementation/Strings/
+    Levenshtein_distance#Python
+    """
+    if len(s1) < len(s2):
+        return _levenshtein(s2, s1)
+    if not s1:
+        return len(s2)
+
+    previous_row = xrange(len(s2) + 1)
+    for i, c1 in enumerate(s1):
+        current_row = [i + 1]
+        for j, c2 in enumerate(s2):
+            insertions = previous_row[j + 1] + 1
+            deletions = current_row[j] + 1
+            substitutions = previous_row[j] + (c1 != c2)
+            current_row.append(min(insertions, deletions, substitutions))
+        previous_row = current_row
+
+    return previous_row[-1]
+
+def _string_dist_basic(str1, str2):
+    """Basic edit distance between two strings, ignoring
+    non-alphanumeric characters and case. Normalized by string length.
+    """
+    str1 = re.sub(r'[^a-z0-9]', '', str1.lower())
+    str2 = re.sub(r'[^a-z0-9]', '', str2.lower())
+    if not str1 and not str2:
+        return 0.0
+    return _levenshtein(str1, str2) / float(max(len(str1), len(str2)))
+
+def ratio(str1, str2):
+    """Gives an "intuitive" edit distance between two strings. This is
+    an edit distance, normalized by the string length, with a number of
+    tweaks that reflect intuition about text.
+    """
+    str1 = str1.lower()
+    str2 = str2.lower()
+
+    # Don't penalize strings that move certain words to the end. For
+    # example, "the something" should be considered equal to
+    # "something, the".
+    for word in SD_END_WORDS:
+        if str1.endswith(', %s' % word):
+            str1 = '%s %s' % (word, str1[:-len(word)-2])
+        if str2.endswith(', %s' % word):
+            str2 = '%s %s' % (word, str2[:-len(word)-2])
+
+    # Change the weight for certain string portions matched by a set
+    # of regular expressions. We gradually change the strings and build
+    # up penalties associated with parts of the string that were
+    # deleted.
+    base_dist = _string_dist_basic(str1, str2)
+    penalty = 0.0
+    for pat, weight in SD_PATTERNS:
+        # Get strings that drop the pattern.
+        case_str1 = re.sub(pat, '', str1)
+        case_str2 = re.sub(pat, '', str2)
+
+        if case_str1 != str1 or case_str2 != str2:
+            # If the pattern was present (i.e., it is deleted in the
+            # the current case), recalculate the distances for the
+            # modified strings.
+            case_dist = _string_dist_basic(case_str1, case_str2)
+            case_delta = max(0.0, base_dist - case_dist)
+            if case_delta == 0.0:
+                continue
+
+            # Shift our baseline strings down (to avoid rematching the
+            # same part of the string) and add a scaled distance
+            # amount to the penalties.
+            str1 = case_str1
+            str2 = case_str2
+            base_dist = case_dist
+            penalty += weight * case_delta
+    dist = base_dist + penalty
+
+    return 1 - dist
+
 
 dirlevels = lambda a: len(a.split('/'))
 
@@ -682,7 +775,7 @@ def load_actions():
 
     funcs = []
     for f in files:
-        action = findfunc.load_action(f)
+        action = findfunc.load_macro_info(f)
         funcs.append([action[0], action[1], f])
     return funcs
 
@@ -716,14 +809,32 @@ def progress(func, pstring, maximum, threadfin = None):
     case where only the parent argument is passed).
     """
     def s(*args):
+
+        focused = QApplication.focusWidget()
+        if focused:
+            focusedpar = focused.parentWidget()
+        else:
+            focusedpar = None
+        
         parent = args[0]
-        win = ProgressWin(parent, maximum, pstring)
+        
         if maximum > 1:
+            win = ProgressWin(parent, maximum, pstring)
             win.show()
+            
         if len(args) > 1:
             f = func(*args)
         else:
             f = func()
+
+        if maximum  == 1:
+            errors = f.next()
+            if errors and \
+                not isinstance(errors, (QString, int, long, basestring)):
+                errormsg(parent, errors[0], 1)
+            if threadfin:
+                threadfin()
+            return
         parent.showmessage = True
 
         def threadfunc():
@@ -737,7 +848,6 @@ def progress(func, pstring, maximum, threadfin = None):
                     elif isinstance(temp, (int, long)):
                         thread.emit(SIGNAL('set_max(int)'), temp)
                     elif temp is not None:
-                        #temp[0] is the error message, temp[1] the num files
                         thread.emit(SIGNAL('error(QString, int)'),
                             temp[0], temp[1])
                         err = True
@@ -753,10 +863,14 @@ def progress(func, pstring, maximum, threadfin = None):
 
         def threadexit(*args):
             if args[0] == -1:
-                win.close()
                 win.destroy()
-                if threadfin:
+                QApplication.processEvents()
+                if threadfin:                    
                     threadfin()
+                if focusedpar is not None:
+                    try: focusedpar.setFocus()
+                    except RuntimeError: pass
+                return
             elif isinstance(args[0], QString):
                 if parent.showmessage:
                     ret = errormsg(parent, args[0], maximum)
@@ -799,8 +913,10 @@ def timemethod(method):
 
 class HeaderSetting(QDialog):
     """A dialog that allows you to edit the header of a TagTable widget."""
-    def __init__(self, tags = None, parent = None, showok = True, showedits = True):
+    def __init__(self, tags=None, parent=None, showok=True, showedits=True):
+
         QDialog.__init__(self, parent)
+
         self.listbox = ListBox()
         self.tags = [list(z) for z in tags]
         self.listbox.addItems([z[0] for z in self.tags])
@@ -808,9 +924,12 @@ class HeaderSetting(QDialog):
         self.vbox = QVBoxLayout()
         self.vboxgrid = QGridLayout()
         self.textname = QLineEdit()
-        self.tag = QLineEdit()
+        self.tag = QComboBox()
+        self.tag.addItems(sorted(INFOTAGS) + gettaglist())
+        self.tag.setEditable(True)
         self.buttonlist = ListButtons()
         self.buttonlist.edit.setVisible(False)
+
         if showedits:
             self.vboxgrid.addWidget(QLabel(translate("Column Settings", "Title")),0,0)
             self.vboxgrid.addWidget(self.textname,0,1)
@@ -830,9 +949,11 @@ class HeaderSetting(QDialog):
         self.grid.setColumnStretch(1,1)
         self.grid.setColumnStretch(0,2)
 
-        self.connect(self.listbox, SIGNAL("currentItemChanged (QListWidgetItem *,QListWidgetItem *)"), self.fillEdits)
-        self.connect(self.listbox, SIGNAL("itemSelectionChanged()"),self.enableEdits)
+        self.connect(self.listbox,
+            SIGNAL("currentItemChanged (QListWidgetItem *,QListWidgetItem *)"),
+            self.fillEdits)
 
+        self.connect(self.listbox, SIGNAL("itemSelectionChanged()"),self.enableEdits)
 
         self.okbuttons = OKCancel()
         if showok is True:
@@ -891,20 +1012,20 @@ class HeaderSetting(QDialog):
         try: #An error is raised if the last item has just been removed
             if row > -1:
                 self.tags[row][0] = unicode(self.textname.text())
-                self.tags[row][1] = unicode(self.tag.text())
+                self.tags[row][1] = unicode(self.tag.currentText())
         except IndexError:
             pass
 
         row = self.listbox.row(current)
         if row > -1:
             self.textname.setText(self.tags[row][0])
-            self.tag.setText(self.tags[row][1])
+            self.tag.setEditText(self.tags[row][1])
 
     def okClicked(self):
         row = self.listbox.currentRow()
         if row > -1:
             self.tags[row][0] = unicode(self.textname.text())
-            self.tags[row][1] = unicode(self.tag.text())
+            self.tags[row][1] = unicode(self.tag.currentText())
         self.emit(SIGNAL("headerChanged"),[z for z in self.tags])
         self.close()
 
@@ -1310,7 +1431,7 @@ class ArtworkLabel(QGraphicsView):
     def dropEvent(self, event):
         mime = event.mimeData()
         if mime.hasUrls():
-            filenames = [unicode(z.path()) for z in mime.urls()]
+            filenames = [unicode(z.toString()) for z in mime.urls()]
             self.emit(SIGNAL('newImages'), *filenames)
         super(ArtworkLabel, self).dropEvent(event)
 
@@ -1403,7 +1524,7 @@ class PicWidget(QWidget):
 
         #Description and picture type shit.
         self._image_desc = QLineEdit(self)
-        self._image_desc.setText(translate("Artwork", 'Enter a description'))
+        self._image_desc.setPlaceholderText(translate("Artwork", 'Enter a description'))
         self._image_desc.setToolTip(
             translate("Artwork",
             '<p>Enter a description for the current cover.</p>'
@@ -1587,12 +1708,11 @@ class PicWidget(QWidget):
             filedlg = QFileDialog()
             filename = unicode(filedlg.getOpenFileName(self,
                 translate("Artwork", 'Select Image...'), default_fn,
-                translate("Artwork", "JPEG Images (*.jpg);;PNG Images (*.png);;All Files(*.*)")))
+                                                       translate("Artwork", "JPEG & PNG Images (*.jpg *.jpeg *.png);;JPEG Images (*.jpg *.jpeg);;PNG Images (*.png);;All Files(*.*)")))
 
         if not filename:
             return
         self.lastfilename = filename
-        data = open(filename, 'rb').read()
         pic = self.loadPics(filename)
         if pic:
             pic = pic[0]
@@ -1608,7 +1728,7 @@ class PicWidget(QWidget):
             self.emit(SIGNAL('imageChanged'))
     
     def addImages(self, images):
-        if not self._itags:
+        if not self._itags or not images:
             return
         if self.images:
             index = len(self.images)
@@ -1790,27 +1910,35 @@ class PicWidget(QWidget):
 
     def loadPics(self, *filenames):
         """Loads pictures from the filenames"""
+        #I really need to sort out these circular references.
+        from puddlestuff.tagsources import RetrievalError, urlopen 
         images = []
+        
         for filename in filenames:
             image = QImage()
-            if image.load(filename):
+            if filename.startswith(":/"):
+                ba = QByteArray()
+                data = QBuffer(ba)
+                data.open(QIODevice.WriteOnly)
+                image.save(data, "JPG")
+                data = str(data.data())
+            else:
                 try:
-                    data = open(filename, 'rb').read()
-                except IOError, e:
-                    if filename.startswith(u':/'):
-                        ba = QByteArray()
-                        data = QBuffer(ba)
-                        data.open(QIODevice.WriteOnly)
-                        image.save(data, "JPG")
-                        data = str(data.data())
-                    else:
-                        raise e
+                    data = urlopen(filename)
+                except (ValueError, RetrievalError):
+                    try:
+                        data = open(filename, 'r').read()
+                    except EnvironmentError:
+                        continue
+
+            if image.loadFromData(data):
                 pic = {'data': data, 'height': image.height(),
                     'width': image.width(), 'size': len(data),
                     'mime': 'image/jpeg',
                     'description': translate("Artwork", 'Enter description'),
                     'imagetype': 3}
                 images.append(pic)
+
         return images
 
     def picsFromData(self, *data):
@@ -1936,6 +2064,8 @@ class ProgressWin(QDialog):
             def update():
                 self.setValue(self.pbar.value() + 1)
             self.connect(self._timer, SIGNAL('timeout()'), update)
+
+        if maximum <= 0:
             self._timer.start()
 
     def setValue(self, value):
@@ -2008,6 +2138,7 @@ class PuddleCombo(QWidget):
         if not default:
             default = []
         self.combo.clear()
+        cparser = PuddleConfig(self.filename)
         self.combo.addItems(cparser.load(self.name, 'values', default))
 
     def save(self):
