@@ -20,8 +20,9 @@ import puddlestuff.webdb
 import loadshortcuts as ls
 import m3u, findfunc, genres
 
-from puddlestuff.puddlesettings import SettingsDialog, load_gen_settings
+from puddlestuff.puddlesettings import SettingsDialog, load_gen_settings, update_settings
 import puddlestuff.mainwin.funcs as mainfuncs
+from puddlestuff.helperwin import ConfirmationErrorDialog
 from functools import partial
 from itertools import izip
 import puddlestuff.audioinfo as audioinfo
@@ -70,11 +71,12 @@ def create_tool_windows(parent, extra=None):
     docks = []
     cparser = PuddleConfig()
     cparser.filename = ls.menu_path
-    controls = [z.control for z in (mainwin.tagpanel, mainwin.artwork,
-        mainwin.dirview,
-        mainwin.patterncombo, mainwin.filterwin, puddlestuff.webdb, 
-        mainwin.storedtags, mainwin.logdialog, 
-        puddlestuff.masstag.dialogs)]
+    widgets = (mainwin.tagpanel, mainwin.artwork,
+        mainwin.dirview, mainwin.patterncombo, mainwin.filterwin,
+        puddlestuff.webdb, mainwin.storedtags, mainwin.logdialog,
+        puddlestuff.masstag.dialogs)
+    
+    controls = [z.control for z in widgets]
     controls.extend(mainwin.action_dialogs.controls)
     if extra:
         controls.extend(extra)
@@ -204,15 +206,36 @@ def connect_action_shortcuts(actions):
 
 def help_menu(parent):
     menu = QMenu(translate("Menus", 'Help'), parent)
+    open_url = lambda url: QDesktopServices.openUrl(QUrl(url))
+
+    connect = lambda c,s: c.connect(c, SIGNAL('triggered()'), s)
+    
+    doc_link = QAction(translate("Menus", 'Online &Documentation'),
+        parent)
+    connect(doc_link, lambda: open_url('http://puddletag.sf.net/docs.html'))
+
+    forum_link = QAction(translate("Menus", '&Forum'), parent)
+    connect(forum_link,
+        lambda: open_url('http://sourceforge.net/apps/phpbb/puddletag'))
+
+    issue_link = QAction(translate("Menus", '&Bug tracker'), parent)
+    connect(issue_link,
+        lambda: open_url('http://code.google.com/p/puddletag/issues'))
+    
     about_icon = get_icon('help-about', QIcon())
     about = QAction(about_icon,
         translate("Menus", 'About puddletag'), parent)
+    connect(about, partial(mainfuncs.show_about, parent))
+    
     about_qt = QAction(translate("Menus", 'About Qt'), parent)
-    about_qt.connect(about_qt, SIGNAL('triggered()'), QApplication.aboutQt)
-    about.connect(about, SIGNAL('triggered()'), 
-        partial(mainfuncs.show_about, parent))
-    menu.addAction(about)
-    menu.addAction(about_qt)
+    connect(about_qt, QApplication.aboutQt)
+    
+
+    sep = QAction(parent)
+    sep.setSeparator(True)
+    map(menu.addAction, (doc_link, forum_link, issue_link, sep,
+        about, about_qt))
+
     return menu
 
 def load_plugins():
@@ -237,6 +260,9 @@ class MainWin(QMainWindow):
     def __init__(self):
         QMainWindow.__init__(self)
 
+        self.__updateDirs = True
+        self.__dirsToUpdate = []
+
         global add_shortcuts
         global remove_shortcuts
         add_shortcuts = self.addShortcuts
@@ -325,7 +351,12 @@ class MainWin(QMainWindow):
 
         for m in plugin_modules:
             if hasattr(m, 'init'):
-                m.init(parent=self)
+                try:
+                    m.init(parent=self)
+                except:
+                    
+                    traceback.print_exc()
+                    continue
 
         for win in plugin_dialogs:
             try:
@@ -387,7 +418,6 @@ class MainWin(QMainWindow):
     def _dirChanged(self, dirs):
         if not dirs:
             self.setWindowTitle('puddletag')
-            self._lastdir = []
             return
         
         if isinstance(dirs, basestring):
@@ -410,6 +440,7 @@ class MainWin(QMainWindow):
             self.setWindowTitle(translate("Main Window", 'puddletag: %1 + others').arg(initial))
         else:
             self.setWindowTitle(translate("Main Window", 'puddletag: %1').arg(initial))
+
         self._lastdir = dirs
 
     def _getDir(self):
@@ -434,6 +465,13 @@ class MainWin(QMainWindow):
         pass
 
     def closeEvent(self, e):
+        preview_msg = translate('Previews',
+            'Some files have uncommited previews. '
+            'These changes will be lost once you exit puddletag. <br />'
+            'Do you want to exit without writing those changes?<br />')
+        if tagmodel.has_previews(parent=self, msg=preview_msg):
+            e.ignore()
+            return False
         controls = PuddleDock._controls
         for control in PuddleDock._controls.values():
             if hasattr(control, 'saveSettings'):
@@ -453,7 +491,6 @@ class MainWin(QMainWindow):
         settings.setValue('table/header', QVariant(headstate))
         genres.save_genres(status['genres'])
         e.accept()
-        #QMainWindow.closeEvent(self, e)
 
     def createStatusBar(self):
         statusbar = self.statusBar()
@@ -557,10 +594,6 @@ class MainWin(QMainWindow):
 
         cparser = PuddleConfig()
         settings = QSettings(constants.QT_CONFIG, QSettings.IniFormat)
-
-        h = self._table.horizontalHeader()
-        h.restoreState(settings.value('table/header').toByteArray())
-        self.restoreState(settings.value('main/state').toByteArray())
         
         gensettings = {}
         controls = PuddleDock._controls.values()
@@ -590,8 +623,8 @@ class MainWin(QMainWindow):
                 u'MusicBrainz Album Id': u'mbrainz_album_id'},
             u'ID3':
                 {u'ufid:http://musicbrainz.org': u'mbrainz_track_id',
-                u'MusicBrainz Album ID': u'mbrainz_album_id',
-                u'MusicBrainz Artist ID': u'mbrainz_artist_id'},
+                u'MusicBrainz Album Id': u'mbrainz_album_id',
+                u'MusicBrainz Artist Id': u'mbrainz_artist_id'},
             u'APEv2':
                 {u'musicbrainz_albumid': u'mbrainz_album_id',
                 u'musicbrainz_artistid': u'mbrainz_artist_id',
@@ -609,14 +642,33 @@ class MainWin(QMainWindow):
         winsettings('mainwin', self)
         if cparser.get("main", "maximized", True):
             self.showMaximized()
+        
         QApplication.processEvents()
+
+        if constants.FS_ENC == "ascii":
+            QMessageBox.warning(self, "puddletag", translate("Errors",
+                "Your filesystem encoding was detected as <b>ASCII</b>. <br />"
+                "You won't be able to rename files using accented, <br />"
+                " cyrillic or any characters outside the ASCII alphabet."))
+        
         for control, val in gensettings.items():
             control.applyGenSettings(val, 1)
+
+        h = self._table.horizontalHeader()
+        h.restoreState(settings.value('table/header').toByteArray())
+        self.restoreState(settings.value('main/state').toByteArray())
+        
         confirmations.load()
         shortcutsettings.ActionEditorDialog._loadSettings(status['actions'])
+        update_settings()
+
+        QApplication.processEvents()
+        
 
     def savePlayList(self):
-        tags = self._table.model().taginfo
+        tags = status['selectedfiles']
+        if not tags:
+            tags = status['alltags']
         settings = PuddleConfig()
         try:
             dirname = self._lastdir[0]
@@ -633,7 +685,8 @@ class MainWin(QMainWindow):
                 pattern = None
 
             reldir = settings.get('playlist', 'reldir', 0, True)
-            m3u.exportm3u(tags, f, pattern, reldir)
+            windows_separator = settings.get('playlist', 'windows_separator', 0, False)
+            m3u.exportm3u(tags, f, pattern, reldir, windows_separator)
 
     def _viewFilled(self, val):
         self.emit(SIGNAL('viewfilled'), val)
@@ -659,7 +712,8 @@ class MainWin(QMainWindow):
     def updateTotalStats(self, *args):
         self._totalstats.setText(self._updateStatus(status['alltags']))
 
-    def _write(self, tagiter, rows=None):
+    def _write(self, tagiter, rows=None, previews=None):
+        self.__updateDirs = False
         if not rows:
             rows = status['selectedrows']
         model = self._table.model()
@@ -671,6 +725,9 @@ class MainWin(QMainWindow):
                 self.emit(SIGNAL('libfilesedited'), lib_updates)
         lib_updates = []
 
+        failed_rows = [rows[0]] #First element=last row used.
+                                #Rest, rows that failed to write.
+
         if model.previewMode:
             [setRowData(row, f, undo=True) for row, f in izip(rows, tagiter)]
             fin()
@@ -678,23 +735,42 @@ class MainWin(QMainWindow):
 
         def func():
             for row, f in izip(rows, tagiter):
+                failed_rows[0] = row
                 try:
                     update = setRowData(row, f, undo=True)
                     if update:
                         lib_updates.append(update)
                     yield None
                 except EnvironmentError, e:
+                    failed_rows.append(row)
                     filename = model.taginfo[row][PATH]
                     m = rename_error_msg(e, filename)
                     if row == rows[-1]:
                         yield m, 1
                     else:
                         yield m, len(rows)
-        
-        return func, fin, rows
 
-    def writeTags(self, tagiter, rows=None):
-        ret = self._write(tagiter, rows)
+        def finished():
+            self.__updateDirs = True
+            self.updateDirs([])
+            if previews and failed_rows[1:]:
+                model.previewMode = True
+                last_row = failed_rows[0]
+                [failed_rows.append(r) for r in previews if r > last_row]
+                taginfo = model.taginfo
+                for row in failed_rows[1:]:
+                    if row not in previews:
+                        continue
+                    taginfo[row].preview = previews[row]
+                last_row = failed_rows[0]
+                
+                model.updateTable(failed_rows)
+            return fin()
+        
+        return func, finished, rows
+
+    def writeTags(self, tagiter, rows=None, previews=None):
+        ret = self._write(tagiter, rows, previews)
         if ret is None:
             return
         
@@ -713,43 +789,19 @@ class MainWin(QMainWindow):
             fin()
             if 'rename_dirs' in state:
                 self.renameDirs(state['rename_dirs'].items())
-        s = progress(func, translate("Defaults", 'Writing '), len(rows), finished)
+        s = progress(func, translate("Defaults", 'Writing '), len(rows),
+            finished)
         s(self)
 
     def writeOneToMany(self, d):
-        model = self._table.model()
         rows = status['selectedrows']
-        setRowData = model.setRowData
-        lib_updates = []
-
-        def fin():
-            model.undolevel += 1
-            self._table.selectionChanged()
-            if not model.previewMode:
-                self.emit(SIGNAL('libfilesedited'), lib_updates)
-
-        if model.previewMode:
-            [setRowData(row, d, undo=True) for row in rows]
-            fin()
+        ret = self._write((d.copy() for r in rows), rows)
+        if ret is None:
             return
+        func, fin, rows = ret
 
-        def func():
-            for row in rows:
-                try:
-                    update = setRowData(row, d, undo=True)
-                    if update:
-                        lib_updates.append(update)
-                    yield None
-                except (IOError, OSError), e:
-                    filename = model.taginfo[row][PATH]
-                    m = translate("Defaults",
-                        'An error occured while writing to <b>%1</b>. (%2)').arg(filename).arg(e.strerror)
-                    if row == rows[-1]:
-                        yield m, 1
-                    else:
-                        yield m, len(rows)
-
-        s = progress(func, translate("Defaults", 'Writing '), len(rows), fin)
+        s = progress(func, translate("Defaults", 'Writing '),
+            len(rows), fin)
         s(self)
     
     def writeSinglePreview(self, d):
@@ -787,16 +839,18 @@ class MainWin(QMainWindow):
 
     def _writePreview(self):
         taginfo = self._table.model().taginfo
-        def get(audio):
-            ret = audio.preview
+        previews = {}
+        def get(audio, row):
+            preview = audio.preview
             audio.preview = {}
-            return ret
-        data = [(row, get(audio)) for row, audio in 
+            previews[row] = preview
+            return row, preview
+        data = [get(audio, row) for row, audio in
                     enumerate(taginfo) if audio.preview]
         if not data:
             return
         self._table.model().previewMode = False
-        self.writeTags((z[1] for z in data), [z[0] for z in data])
+        self.writeTags((z[1] for z in data), [z[0] for z in data], previews)
 
     def _renameSelected(self, filenames):
         rows = status['selectedrows']
@@ -866,6 +920,23 @@ class MainWin(QMainWindow):
         self._table.restoreSelection()
     
     def updateDirs(self, dirs):
+        if self.__updateDirs:
+            if self.__dirsToUpdate:
+                dirs = self.__dirsToUpdate + dirs
+            self.__dirsToUpdate = []
+        else:
+           self.__dirsToUpdate.extend(dirs)
+           return
+
+        old_dirs = set()
+        new_dirs = []
+        for old_dir, new_dir in dirs:
+            if old_dir not in old_dirs:
+                new_dirs.append([old_dir, new_dir])
+                old_dirs.add(old_dir)
+
+        dirs = new_dirs
+
         if self._lastdir:
             last = self._lastdir[::]
         else:
