@@ -29,6 +29,8 @@ from puddlestuff.translations import translate
 from puddlestuff.util import rename_error_msg
 from puddlestuff.audio_filter import parse as filter_audio
 
+the_break = False
+
 status = {}
 
 SETDATAERROR = SIGNAL("setDataError")
@@ -68,7 +70,8 @@ def loadsettings(filepath=None):
     tags = settings.get('tableheader', 'tags',
         ['__filename', 'artist', 'title','album', 'track', 
         '__length', 'year', '__bitrate','genre', 'comment', '__dirpath'])
-    checked = settings.get('tableheader', 'enabled', range(len(tags)), True)
+    #checked = settings.get('tableheader', 'enabled', range(len(tags)), True)
+    #checked = []
     fontsize = settings.get('table', 'fontsize', 0, True)
     rowsize = settings.get('table', 'rowsize', -1, True)
     v1_option = settings.get('id3tags', 'v1_option', 2)
@@ -77,7 +80,7 @@ def loadsettings(filepath=None):
     audioinfo.id3.v2_option = v2_option
     filespec = u';'.join(settings.get('table', 'filespec', []))
 
-    return ((zip(titles, tags), checked), fontsize, rowsize, filespec)
+    return (zip(titles, tags), fontsize, rowsize, filespec)
 
 def caseless(tag, audio):
     if tag in audio:
@@ -88,6 +91,42 @@ def caseless(tag, audio):
     except IndexError:
         return tag
 
+def has_previews(tags=None, parent=None, msg=None):
+    """Disables preview mode. Returns True if disable successful, else False.
+
+    Displays a confirmation to the user (message box) asking if they want
+    to exit Preview Mode if any files contain previews. If the user selects
+    that they want to exit, then True is returned. Preview Mode is not
+    changed.
+
+    If not previews in tags, then True is returned.
+
+    tags is a list of audioinfo.Tag object with the preview attribute.
+        If preview not empty, the file will be considered to have a preview.
+        If not specified status['alltags'] is used.
+    parent => A window to have as the parent for the message box.
+    msg => Message to show in message box.
+    """
+
+    if msg is None:
+        msg = translate("Previews", 'Do you want to exit Preview Mode?')
+    
+    if tags is None:
+        tags = status['alltags']
+
+    previews = False
+    for z in tags:
+        if z.preview:
+            previews = True
+            break
+
+    if previews and confirmations.should_show('preview_mode'):
+        ret = QMessageBox.question(parent, 'puddletag', msg,
+            QMessageBox.Yes, QMessageBox.No)
+        if ret != QMessageBox.Yes:
+            return True
+    return False
+
 def tag_in_file(tag, audio):
     if tag in audio:
         return tag
@@ -321,27 +360,49 @@ class Properties(QDialog):
 class ColumnSettings(HeaderSetting):
     """A dialog that allows you to edit the header of a TagTable widget."""
     title = translate("Column Settings", 'Columns')
-    def __init__(self, parent = None, showok = False, status=None):
-        (self.tags, checked), fontsize, rowsize, filespec = loadsettings()
+    def __init__(self, parent = None, showok=False, status=None, table=None):
+
+        self.tags, fontsize, rowsize, filespec = loadsettings()
+
+        checked = []
+
+        if table is None and status is not None:
+            table = status['table']
+
+        if table is not None:
+            
+            
+            tags = table.model().headerdata
+            hd = table.horizontalHeader()
+
+            tags = ([hd.visualIndex(i), z, hd.isSectionHidden(i)] for i, z in
+                enumerate(tags))
+            tags = sorted(tags)
+            checked = [i for i, t in enumerate(tags) if not t[2]]
+            tags = [tuple(z[1]) for z in tags]
+            [tags.append(z) for z in self.tags if z not in tags]
+            self.tags = tags
+
         HeaderSetting.__init__(self, self.tags, parent, showok, True)
+        
         if showok:
             winsettings('columnsettings', self)
-        self.buttonlist.moveup.hide()
-        self.buttonlist.movedown.hide()
+        
         self.setWindowFlags(Qt.Widget)
         label = QLabel(translate("Column Settings", 'Adjust visibility of columns.'))
         self.grid.addWidget(label, 0, 0)
         items = [self.listbox.item(z) for z in range(self.listbox.count())]
+        
         if not checked:
-            checked = []
+            checked = range(len(tags))
         [z.setCheckState(Qt.Checked) if i in checked
             else z.setCheckState(Qt.Unchecked) for i,z in enumerate(items)]
 
-    def applySettings(self, control = None):
+    def applySettings(self, control = None):        
         row = self.listbox.currentRow()
         if row > -1:
             self.tags[row][0] = unicode(self.textname.text())
-            self.tags[row][1] = unicode(self.tag.text())
+            self.tags[row][1] = unicode(self.tag.currentText())
         checked = [z for z in range(self.listbox.count()) if
             self.listbox.item(z).checkState()]
         titles = [z[0] for z in self.tags]
@@ -349,15 +410,17 @@ class ColumnSettings(HeaderSetting):
         cparser = PuddleConfig()
         cparser.set('tableheader', 'titles', titles)
         cparser.set('tableheader', 'tags', tags)
-        cparser.set('tableheader', 'enabled', checked)
-
-        headerdata = [z for i, z in enumerate(self.tags) if i in checked]
+        
 
+        hidden = [i for i, z in enumerate(self.tags) if i not in checked]
+        
         if control:
-            control.setHeaderTags(headerdata)
+            control.horizontalHeader().reset()
+            control.setHeaderTags(self.tags, hidden)
             control.restoreSelection()
+            control.horizontalHeader().reset()
         else:
-            self.emit(SIGNAL("headerChanged"), headerdata)
+            self.emit(SIGNAL("headerChanged"), self.tags, hidden)
 
     def add(self):
         HeaderSetting.add(self)
@@ -394,6 +457,7 @@ class TagModel(QAbstractTableModel):
 
         QAbstractTableModel.__init__(self)
         self.alternateAlbumColors = True
+        self._headerData = []
         self.headerdata = headerdata
         self.colorRows = []
         self.sortOrder = (0, Qt.AscendingOrder)
@@ -431,9 +495,6 @@ class TagModel(QAbstractTableModel):
                 z.library = None
         self.undolevel = 0
         self._fontSize = QFont().pointSize()
-        
-        self.columns = dict([(field, i) for i, 
-            (title, field) in enumerate(headerdata)])
 
     def _setFontSize(self, size):
         self._fontSize = size
@@ -447,6 +508,28 @@ class TagModel(QAbstractTableModel):
 
     fontSize = property(_getFontSize, _setFontSize)
 
+    def _getHeaderData(self):
+        return self._headerData
+
+    def _setHeaderData(self, tags):
+
+        new_tags = []
+        indexes = {}
+
+        for i, t in enumerate(tags):
+            t = tuple(t)
+            if t not in indexes:
+                indexes[t] = i
+                new_tags.append(t)
+
+        self._headerData = new_tags
+
+        self.columns = dict((field, i) for i, (title, field) in
+            enumerate(new_tags))
+
+
+    headerdata = property(_getHeaderData, _setHeaderData)
+
     def _get_pBg(self):
         return self._previewBackground
 
@@ -703,6 +786,26 @@ class TagModel(QAbstractTableModel):
         #self.emit(SIGNAL('modelReset')) #Because of the strange behaviour mentioned in reset.
         return True
 
+    def reloadTags(self, tags):
+        for z in tags:
+            z.preview = {}
+            z.undo = {}
+            z.previewundo = {}
+            z.color = None
+            z._temp = {}
+            if not hasattr(z, 'library'):
+                z.library = None
+
+        fns = dict((f.filepath, i) for i,f in enumerate(self.taginfo))
+
+        to_append = []
+
+        for t in tags:
+            if t.filepath in fns:
+                self.taginfo[fns[t.filepath]] = t
+            else:
+                self.taginfo.append(t)
+
     def load(self, taginfo, headerdata=None, append = False):
         """Loads tags as in __init__.
         If append is true, then the tags are just appended."""
@@ -745,7 +848,6 @@ class TagModel(QAbstractTableModel):
             self.undolevel = 0
 
     def moveRows(self, rows, row):
-        print rows, row
 
         taginfo = self.taginfo
         rows = sorted(rows)
@@ -887,7 +989,7 @@ class TagModel(QAbstractTableModel):
     def rowCount(self, index = QModelIndex()):
         return len(self.taginfo)
 
-    def setData(self, index, value, role = Qt.EditRole):
+    def setData(self, index, value, role = Qt.EditRole, dontwrite=False):
         """Sets the data of the currently edited cell as expected.
         Also writes tags and increases the undolevel."""
         if index.isValid() and 0 <= index.row() < len(self.taginfo):
@@ -901,8 +1003,10 @@ class TagModel(QAbstractTableModel):
 
             if tag not in FILETAGS:
                 newvalue = filter(None, newvalue.split(SEPARATOR))
-            if newvalue == currentfile.get(tag, u''):
+            if newvalue == currentfile.get(tag, u'') and not dontwrite:
                 return False
+            if dontwrite:
+                return {tag: newvalue}, index.row()
             ret = self.setRowData(index.row(), {tag: newvalue}, undo=True)
             if not self.previewMode and currentfile.library:
                 self.emit(SIGNAL('libfilesedited'), [ret])
@@ -914,14 +1018,15 @@ class TagModel(QAbstractTableModel):
     def setHeaderData(self, section, orientation, value, role = Qt.EditRole):
         if (orientation == Qt.Horizontal) and (role == Qt.DisplayRole):
             self.headerdata[section] = value
-        self.columns = dict([(field, i) for i, (title, field) in 
-            enumerate(self.headerdata)])
+
+        self.headerdata = self.headerdata #make sure columns are set
+
         self.emit(SIGNAL("headerDataChanged (Qt::Orientation,int,int)"), orientation, section, section)
 
     def setHeader(self, tags):
         self.headerdata = tags
-        self.columns = dict([(field, i) for i, (title, field) in 
-            enumerate(self.headerdata)])
+        self.emit(SIGNAL("headerDataChanged(Qt::Orientation,int,int)"), 
+            Qt.Horizontal, 0, len(self.headerdata))
         self.reset()
 
     def setRowData(self,row, tags, undo = False, justrename = False, temp=False):
@@ -958,9 +1063,11 @@ class TagModel(QAbstractTableModel):
             undo_val = write(audio, tags, self.saveModification, justrename)
             if undo and undo_val:
                 self._addUndo(audio, undo_val)
+
             if DIRNAME in tags or DIRPATH in tags:
                 self.changeFolder(oldpath, audio.dirpath)
-                self.emit(SIGNAL('dirsmoved'), [[oldpath, audio.dirpath]])
+                self.emit(SIGNAL('dirsmoved'),
+                    [[oldpath, audio.dirpath]])
             if justrename and audio.library:
                 audio.save(True)
             ret = (artist, tags)
@@ -1081,7 +1188,7 @@ class TagModel(QAbstractTableModel):
             else:
                 if audio.library:
                     oldfiles.append(deepcopy(audio.tags))
-                edited.append(self.setRowData(row, undo_tags))
+                edited.append(self.setRowData(row, undo_tags, False))
 
         del(undo[level])
         if rows:
@@ -1158,8 +1265,10 @@ class TableHeader(QHeaderView):
 
     def contextMenuEvent(self, event):
         menu = QMenu(self)
-        settings = menu.addAction(translate("Column Settings", "&Select Columns"))
+        settings = menu.addAction(
+            translate("Column Settings", "&Select Columns"))
         self.connect(settings, SIGNAL('triggered()'), self.setTitles)
+        
         menu.exec_(event.globalPos())
 
     def mousePressEvent(self,event):
@@ -1169,13 +1278,11 @@ class TableHeader(QHeaderView):
         QHeaderView.mousePressEvent(self, event)
 
     def setTitles(self):
-        self.win = ColumnSettings(showok = True)
+        self.win = ColumnSettings(showok=True, table=self.parent())
         self.win.setModal(True)
         self.win.show()
-        self.connect(self.win, SIGNAL("headerChanged"), self.headerChanged)
-
-    def headerChanged(self, val):
-        self.emit(SIGNAL("headerChanged"), val)
+        self.connect(self.win, SIGNAL("headerChanged"),
+            SIGNAL('headerChanged'))
 
 class VerticalHeader(QHeaderView):
     def __init__(self, orientation, parent = None):
@@ -1211,12 +1318,8 @@ class TagTable(QTableView):
         header.setSortIndicatorShown(True)
         self.setSortingEnabled(True)
         self._savedSelection = False
-        #header.setStretchLastSection(True)
 
         self.setVerticalHeader(VerticalHeader(Qt.Vertical))
-
-        #self.connect(header, SIGNAL("sectionClicked(int)"), self.sortByColumn)
-        #header.setSortIndicator(0, Qt.AscendingOrder)
         self.setHorizontalHeader(header)
         self.setDragEnabled(True)
         self.setAcceptDrops(True)
@@ -1249,10 +1352,9 @@ class TagTable(QTableView):
         self.connect(model, SIGNAL('libfilesedited'), SIGNAL('libfilesedited'))
         self.undo = model.undo
 
-        if logging.getLogger().getEffectiveLevel() != logging.DEBUG:
-            self.closeEditor = self._closeEditor
-            delegate = TagDelegate(self)
-            self.setItemDelegate(delegate)
+        self.closeEditor = self._closeEditor
+        delegate = TagDelegate(self)
+        self.setItemDelegate(delegate)
             
         self.subFolders = False
 
@@ -1264,7 +1366,7 @@ class TagTable(QTableView):
         self.emits = ['dirschanged', SELECTIONCHANGED, 'filesloaded',
             'viewfilled', 'filesselected', 'enableUndo',
             'playlistchanged', 'deletedfromlib', 'libfilesedited',
-            'previewModeChanged']
+            'previewModeChanged', 'onetomany']
         self.receives = [
             ('loadFiles', self.loadFiles),
             ('removeFolders', self.removeFolders),
@@ -1315,14 +1417,21 @@ class TagTable(QTableView):
         self._resize = value
         if value:
             self.resizeColumnsToContents()
+            self.connect(self.model(), SIGNAL('modelReset'),
+                self.resizeColumnsToContents)
+        else:
+            self.disconnect(self.model(), SIGNAL('modelReset'),
+                self.resizeColumnsToContents)
 
     def _getSelectedTags(self):
         columns = dict([(v,k) for k,v in self.model().columns.items()])
         rows = self.currentRowSelection()
         audios = self.selectedTags
 
+
         def get_selected(f, row):
             selected = (columns[column] for column in rows[row])
+                
             return ((field, f.get(field, u'')) for field in selected)
         
         return map(dict, (get_selected(*z) for z in zip(audios, sorted(rows))))
@@ -1388,8 +1497,10 @@ class TagTable(QTableView):
             model.emit(SETDATAERROR,
                 rename_error_msg(editor.writeError, currentfile.filepath))
             return
-        
-        if not editor.returnPressed:
+
+        if len(self.selectedRows) > 1:
+            QTableView.closeEditor(self, editor, QAbstractItemDelegate.NoHint)
+        elif not editor.returnPressed:
             QTableView.closeEditor(self, editor, hint)
         else:
             index = self.currentIndex()
@@ -1437,10 +1548,14 @@ class TagTable(QTableView):
                     yield translate("Table", "There was an error deleting the "
                         "tag of %1: <b>Tag deletion isn't supported"
                         "for %2 files.</b>").arg(filename).arg(ext), rowlen
-            self.model().undolevel += 1
-            self.selectionChanged()
 
-        f = progress(func, translate("Table", 'Deleting tag... '), len(self.selectedRows))
+
+        def fin():
+            self.selectionChanged()
+            self.model().undolevel += 1
+
+        f = progress(func, translate("Table", 'Deleting tag... '),
+            len(self.selectedRows), fin)
         f(self)
 
     def columnCount(self):
@@ -1609,10 +1724,14 @@ class TagTable(QTableView):
         """
         self.selectedRows = []
         self.selectedColumns = []
-        self.model().load(tags, append = append)
+        if append is None:
+            self.model().reloadTags(tags)
+        else:
+            self.model().load(tags, append=append)
 
-        if append:
-            self.restoreSelection()
+        QApplication.processEvents()
+        if append or append is None:
+            self.restoreSelection() 
         else:
             self.selectCorner()
 
@@ -1653,7 +1772,7 @@ class TagTable(QTableView):
         QTableView.keyPressEvent(self, event)
 
     def loadFiles(self, files=None, dirs=None, append=False, subfolders=None,
-                filepath=None):
+                filepath=None, post_process=None):
         assert files or dirs, 'Either files or dirs (or both) must be specified.'
 
         if subfolders is None:
@@ -1703,8 +1822,6 @@ class TagTable(QTableView):
                 'Reading Directory: %1 + others').arg(dirs[0])
         else:
             reading_dir = translate('Defaults', 'Reading Dir')
-        finished = lambda: self._loadFilesDone(tags, append, filepath)
-
         
         def load_dir():
             if files:
@@ -1726,7 +1843,12 @@ class TagTable(QTableView):
                     tags.append(tag)
                 yield None
 
-        s = progress(load_dir, translate("Defaults", 'Loading '), 20, finished)
+        if post_process:
+            s = progress(load_dir, translate("Defaults", 'Loading '), 20,
+                lambda: post_process(tags, append, filepath))
+        else:
+            s = progress(load_dir, translate("Defaults", 'Loading '), 20,
+                lambda: self._loadFilesDone(tags, append, filepath))
         s(self.parentWidget())
 
     def _loadFilesDone(self, tags, append, filepath):
@@ -1737,9 +1859,7 @@ class TagTable(QTableView):
             self.emit(SIGNAL('playlistchanged'), filepath)
         self.emit(SIGNAL('filesloaded'), True)
         if self._restore:
-            self.clearSelection()
-            self.restoreReloadSelection(*self._restore)
-            self._restore = False
+            self.restoreSelection(self._restore)
 
         model = self.model()
         model.emit(SIGNAL("dataChanged(QModelIndex,QModelIndex)"),
@@ -1756,8 +1876,8 @@ class TagTable(QTableView):
         QTableView.sortByColumn(self, sortcolumn)
 
     def loadSettings(self):
-        ((tags, checked), fontsize, rowsize, self.filespec) = loadsettings()
-        self.setHeaderTags([z for i, z in enumerate(tags) if i in checked])
+        (tags, fontsize, rowsize, self.filespec) = loadsettings()
+        self.setHeaderTags(tags)
         if fontsize:
             self.fontSize = fontsize
         if rowsize > -1:
@@ -1867,21 +1987,48 @@ class TagTable(QTableView):
     
     def previewMode(self, value):
         if not value:
-            previews = False
-            for z in self.model().taginfo:
-                if z.preview:
-                    previews = True
-                    continue
-
-            if confirmations.should_show('preview_mode') and previews:
-                ret = QMessageBox.question(self, 'puddletag',
-                    translate("Previews", 'Do you want to exit Preview Mode?'),
-                    QMessageBox.Yes, QMessageBox.No)
-                if ret != QMessageBox.Yes:
-                    return
+            if has_previews(self.model().taginfo, self):
+                return False
         self.model().previewMode = value
+        return value
 
-    def reloadFiles(self, filenames = None):
+    def reloadSelected(self, files = None):
+        self._restore = self.saveSelection()
+
+        loaded_dirs = map(encode_fn, self.dirs)
+        if files is None:
+            files = self.model().taginfo
+
+        taginfo = self.model().taginfo
+        dirs = set(map(lambda i: taginfo[i].dirpath, self.selectedRows))
+
+        is_sub = lambda fn: filter(None,
+            [issubfolder(z,fn) for z in dirs])
+
+        sub_files = []
+        for f in files:
+            if f.dirpath in dirs or is_sub(f.dirpath):
+                sub_files.append(f)
+
+        self.loadFiles(None, dirs, False, self.subFolders,
+            self._playlist,
+            lambda t,a,f: self.__processReload(t,a,f, sub_files))
+
+    def __processReload(self, tags, append, filepath, sub_files):
+        new_fns = set(z.filepath for z in tags)
+        to_remove = set(z for z in sub_files
+            if z.filepath not in new_fns)
+        self.model().taginfo = [z for z in self.model().taginfo if
+            z not in to_remove]
+        self.fillTable(tags, None)
+        model = self.model().reset()
+        self.emit(SIGNAL('filesloaded'), True)
+        if self._restore:
+            self.restoreSelection(self._restore)
+
+        self.selectionChanged()
+
+    def reloadFiles(self, files=None):
         self._restore = self.saveSelection()
         dirs = map(encode_fn, self.dirs)
         files = [z.filepath for z in self.model().taginfo if z.dirpath
@@ -1927,6 +2074,18 @@ class TagTable(QTableView):
         self.connect(model, SIGNAL('previewModeChanged'), 
             SIGNAL('previewModeChanged'))
         self.connect(model, SIGNAL('dirsmoved'), SIGNAL('dirsmoved'))
+        set_data = model.setData
+
+        def modded_setData(index, value, role=Qt.EditRole):
+            if len(self.selectedRows) == 1:
+                return set_data(index, value, role)
+            ret = set_data(index, value, role, True)
+            
+            if ret:
+                self.emit(SIGNAL('onetomany'), ret[0])
+            return False
+
+        model.setData = modded_setData
 
     def currentRowSelection(self):
         """Returns a dictionary with the currently selected rows as keys.
@@ -2009,15 +2168,19 @@ class TagTable(QTableView):
     def restoreSort(self):
         self.setCursor(Qt.ArrowCursor)
         if self._savedSelection:
-            self.restoreSelection(*self._savedSelection)
+            self.restoreSelection(self._savedSelection)
             self._savedSelection = None
             return
 
     def saveSelection(self):
-        self._currentcol = self.currentColumnSelection()
-        self._currentrow = self.currentRowSelection()
-        self._currenttags = [(row, self.rowTags(row)) for row in self._currentrow]
-        return (self._currentrow, self._currentcol, self._currenttags)
+        taginfo = self.model().taginfo
+        fields = [field for title, field in self.model().headerdata]
+        filenames = defaultdict(lambda: set())
+        selection = self.currentRowSelection()
+        for row, columns in self.currentRowSelection().iteritems():
+            filenames[taginfo[row].filepath] = set(fields[c] for c in columns)
+        self.__savedSelection = filenames
+        return filenames
 
     def restoreReloadSelection(self, currentrow, currentcol, tags):
         if not tags:
@@ -2066,55 +2229,32 @@ class TagTable(QTableView):
             [select(min(row), max(row), col) for row in rows]
         self.selectionModel().select(selection, QItemSelectionModel.Select)
 
-    def restoreSelection(self, currentrow=None, currentcol=None, tags=None):
-        if not currentrow:
-            currentrow = self._currentrow
-            currentcol = self._currentcol
-            tags = self._currenttags
-        if not tags:
-            return
-
-        def getGroups(rows):
-            groups = []
-            try:
-                last = [rows[0]]
-            except IndexError:
-                return []
-            for row in rows[1:]:
-                if row - 1 == last[-1]:
-                    last.append(row)
-                else:
-                    groups.append(last)
-                    last = [row]
-            groups.append(last)
-            return groups
-
-        getrow = self.model().taginfo.index
-        modelindex = self.model().index
+    def restoreSelection(self, data=None):
+        if data is None:
+            data = self.__savedSelection
+            
+        get_index = self.model().index
         selection = QItemSelection()
-        select = lambda top, low, col: selection.append(
-            QItemSelectionRange(modelindex(top, col),
-                modelindex(low, col)))
-        newindexes = {}
 
-        while True:
-            try:
-                tag = tags[0]
-            except IndexError:
-                break
-            try:
-                newindexes[tag[0]] = getrow(tag[1])
-            except ValueError:
-                pass
-            del(tags[0])
-        groups = {}
-        for col, rows in currentcol.items():
-            groups[col] = getGroups(sorted([newindexes[row] for row in rows if row in newindexes]))
+        def select_index(row, col):
+            model_index = get_index(row, col)
+            selection.select(model_index, model_index)
+
+        columns = dict((field[1], i)  for i, field in 
+            enumerate(self.model().headerdata))
+        
+        for row, fn in enumerate(z.filepath for z in self.model().taginfo):
+            selected_fields = data.get(fn)
+            if not selected_fields:
+                continue
+            
+            for field in selected_fields:
+                column = columns.get(field)
+                if column is not None:
+                    select_index(row, column)
 
-        for col, rows in groups.items():
-            [select(min(row), max(row), col) for row in rows]
         self.selectionModel().clear()
-        self.selectionModel().select(selection, QItemSelectionModel.Select)
+        self.selectionModel().select(selection, QItemSelectionModel.SelectCurrent)
         self.model().highlight(self.selectedRows)
 
     def removeFolders(self, dirs, valid = True):
@@ -2122,9 +2262,20 @@ class TagTable(QTableView):
             self.dirs = list(set(self.dirs).difference(dirs))
             self.model().removeFolders(dirs, valid)
 
-    def setHeaderTags(self, tags):
+    def setHeaderTags(self, tags, hidden=None):
+
         self.saveSelection()
+        hd = TableHeader(Qt.Horizontal, tags, self)
+        hd.setSortIndicatorShown(True)
+        hd.setVisible(True)
+        self.connect(hd, SIGNAL("headerChanged"), self.setHeaderTags)
+        self.setHorizontalHeader(hd)
         self.model().setHeader(tags)
+
+        if hidden is not None:
+            for c in hidden:
+                hd.hideSection(c)
+
         self.restoreSelection()
 
     def setHorizontalHeader(self, header):
