@@ -13,13 +13,16 @@ import puddlestuff
 
 from puddlestuff.releasewidget import ReleaseWidget
 import puddlestuff.audioinfo as audioinfo
-from puddlestuff.constants import TEXT, COMBO, CHECKBOX, RIGHTDOCK, SAVEDIR
-from puddlestuff.findfunc import getfunc, replacevars
+from puddlestuff.constants import (TEXT, COMBO, SPINBOX,
+    CHECKBOX, RIGHTDOCK, SAVEDIR)
+from puddlestuff.findfunc import parsefunc
+from puddlestuff.functions import replace_regex
 from puddleobjects import (create_buddy, unique, winsettings,
     ListBox, ListButtons, OKCancel, PuddleConfig, PuddleThread)
 from puddlestuff.tagsources import (tagsources, status_obj, set_useragent,
-    write_log, RetrievalError, mp3tag)
-from puddlestuff.util import pprint_tag, split_by_field, to_string, translate
+    write_log, RetrievalError, mp3tag, SubmissionError)
+from puddlestuff.util import (isempty, pprint_tag,
+    split_by_field, to_string, translate)
 
 pyqtRemoveInputHook()
 
@@ -45,6 +48,27 @@ FIELDLIST_TIP = translate("WebDB",
     '__image</b> will write all fields but the '
     'composer and __image fields.')
 
+DEFAULT_REGEXP = {'album': [u'(.*?)([\(\[\{].*[\)\]\}])', u'$1']}
+
+def apply_regexps(audio, regexps=None):
+    if regexps is None:
+        regexps = DEFAULT_REGEXP
+    audio = deepcopy(audio)
+    changed = False
+    for field, (regexp, output) in regexps.iteritems():
+        if field not in audio:
+            continue
+        text = to_string(audio[field])
+        try:
+            val = replace_regex(audio, text, regexp, output)
+            if val:
+                audio[field] = val
+                if not changed and val != text:
+                    changed = val
+        except puddlestuff.findfunc.FuncError:
+            continue
+    return changed, audio
+
 def display_tag(tag):
     """Used to display tags in in a human parseable format."""
     tag = dict((k,v) for k,v in tag.iteritems() if
@@ -58,9 +82,6 @@ def display_tag(tag):
         text = text[:-len(u'<br />')]
     return text
 
-def display(pattern, tags):
-    return replacevars(getfunc(pattern, tags), audioinfo.stringtags(tags))
-
 def load_mp3tag_sources(dirpath=MTAG_SOURCE_DIR):
     "Loads Mp3tag tag sources from dirpath and return the tag source classes."
     files = glob.glob(os.path.join(dirpath, '*.src'))
@@ -75,7 +96,7 @@ def load_mp3tag_sources(dirpath=MTAG_SOURCE_DIR):
             continue
     return classes
 
-def strip(audio, field_list, reverse = False):
+def strip(audio, field_list, reverse = False, leave_exact=False):
     '''Returns dict of key/values from audio where the key is in field_list.
 
     If reverse is True then the dict will consist of all
@@ -87,8 +108,11 @@ def strip(audio, field_list, reverse = False):
     Any fields starting with '#' will be removed.
     '''
     if not field_list:
-        return dict([(key, audio[key]) for key in audio if 
+        ret = dict([(key, audio[key]) for key in audio if
             not key.startswith('#')])
+        if leave_exact and '#exact' in audio:
+            ret['#exact'] = audio['#exact']
+        return ret
     tags = field_list[::]
     if tags and tags[0].startswith('~'):
         reverse = True
@@ -96,12 +120,16 @@ def strip(audio, field_list, reverse = False):
     else:
         reverse = False
     if reverse:
-        return dict([(key, audio[key]) for key in audio if key not in
+        ret = dict([(key, audio[key]) for key in audio if key not in
                         tags and not key.startswith('#')])
     else:
-        return dict([(key, audio[key]) for key in field_list
+        ret = dict([(key, audio[key]) for key in field_list
             if not key.startswith('#') and key in audio])
 
+    if leave_exact and '#exact' in audio:
+        ret['#exact'] = audio['#exact']
+    return ret
+
 def split_strip(stringlist):
     '''Splits and strips each comma-delimited string in a list of strings.
 
@@ -187,10 +215,7 @@ class SimpleDialog(QDialog):
         for desc, ctype, default in controls:
             if ctype == TEXT:
                 control = QLineEdit(default)
-                label = QLabel(desc)
-                label.setBuddy(control)
-                vbox.addWidget(label)
-                vbox.addWidget(control)
+                vbox.addLayout(create_buddy(desc, control))
             elif ctype == COMBO:
                 control = QComboBox()
                 control.addItems(default[0])
@@ -206,6 +231,13 @@ class SimpleDialog(QDialog):
                 else:
                     control.setCheckState(Qt.Unchecked)
                 vbox.addWidget(control)
+            elif ctype == SPINBOX:
+                control = QSpinBox()
+                control.setMinimum(default[0])
+                control.setMaximum(default[1])
+                control.setValue(default[2])
+                vbox.addLayout(create_buddy(desc, control))
+                
             self._controls.append(control)
         okcancel = OKCancel()
         self.connect(okcancel, SIGNAL('ok'), self.okClicked)
@@ -223,6 +255,8 @@ class SimpleDialog(QDialog):
                 values.append(control.currentIndex())
             elif isinstance(control, QCheckBox):
                 values.append(control.isChecked())
+            elif isinstance(control, QSpinBox):
+                values.append(control.value())
         self.emit(SIGNAL('editingFinished'), values)
         self.close()
 
@@ -487,10 +521,39 @@ class SettingsDialog(QWidget):
 
 def load_source_prefs(name, preferences):
     cparser = PuddleConfig(TAGSOURCE_CONFIG)
-    return [cparser.get(name, option[0], option[2]) if 
-        option[1] != COMBO else 
-        cparser.get(name, option[0], option[2][1]) for 
-        option in preferences]
+    ret = []
+    for option in preferences:
+        if option[1] == COMBO:
+            ret.append(cparser.get(name, option[0], option[2][1]))
+        elif option[1] == SPINBOX:
+            ret.append(cparser.get(name, option[0], option[2][2]))
+        else:
+            ret.append(cparser.get(name, option[0], option[2]))
+    return ret
+
+def tag_source_search(ts, group, files):
+    """Helper method for tag source searches."""
+
+    if not ts.group_by:
+        return ts.search(files), files
+
+    ret = []
+
+    for primary in group:
+        albums = ts.search(primary, group[primary])
+        if albums:
+            ret.extend(albums)
+            continue
+
+        audio = {'album': primary}
+        changed, audio = apply_regexps(audio)
+        if changed:
+            audio['album'] = audio['album'].strip()
+            write_log(translate('WebDB', 'Retrying search with %s') %
+                audio['album'])
+            ret.extend(ts.search(audio['album'], group[primary]))
+
+    return ret, files
 
 class MainWin(QWidget):
     def __init__(self, status, parent = None):
@@ -513,7 +576,10 @@ class MainWin(QWidget):
 
         for ts in self.__sources:
             if hasattr(ts, 'preferences') and not isinstance(ts, QWidget):
-                ts.applyPrefs(load_source_prefs(ts.name, ts.preferences))
+                try:
+                    ts.applyPrefs(load_source_prefs(ts.name, ts.preferences))
+                except:
+                    continue
 
         status['initialized_tagsources'] = self.__sources
 
@@ -542,6 +608,10 @@ class MainWin(QWidget):
         self.searchButton.setAutoDefault(True)
         connect(self.searchButton, "clicked()", self.search)
 
+        self.submitButton = QPushButton(translate("WebDB",
+            "S&ubmit Tags"))
+        connect(self.submitButton, "clicked()", self.submit)
+
         write_preview = QPushButton(translate("WebDB", '&Write'))
         connect(write_preview, "clicked()", self.writePreview)
         
@@ -558,7 +628,7 @@ class MainWin(QWidget):
             'Update empty fields only.'))
         connect(self.listbox, 'statusChanged', self.label.setText)
         connect(self.listbox, 'preview', self.emit_preview)
-        connect(self.listbox, 'exact', self.emitExact)
+        connect(self.listbox, 'exactMatches', self.emitExact)
 
         self.__autoRetrieve = QCheckBox(translate("WebDB",
             'Automatically retrieve matches.'))
@@ -579,8 +649,8 @@ class MainWin(QWidget):
         sourcebox.addWidget(preferences)
 
         hbox = QHBoxLayout()
-        hbox.addWidget(self.searchEdit, 1)
         hbox.addWidget(self.searchButton, 0)
+        hbox.addWidget(self.searchEdit, 1)
 
         vbox = QVBoxLayout()
         vbox.addLayout(sourcebox)
@@ -591,6 +661,7 @@ class MainWin(QWidget):
         hbox = QHBoxLayout()
         hbox.addWidget(infolabel, 1)
         hbox.addStretch()
+        hbox.addWidget(self.submitButton)
         hbox.addWidget(write_preview)
         hbox.addWidget(clear)
         vbox.addLayout(hbox)
@@ -627,6 +698,9 @@ class MainWin(QWidget):
 
         self.searchEdit.setEnabled(hasattr(self.curSource, 'keyword_search'))
 
+        self.submitButton.setVisible(hasattr(self.curSource, 'submit'))
+            
+
     def configure(self):
         config = getattr(self.curSource, 'preferences', None)
         if config is None:
@@ -638,10 +712,15 @@ class MainWin(QWidget):
             defaults = load_source_prefs(self.curSource.name, config)
             prefs = deepcopy(config)
             for pref, value in zip(prefs, defaults):
-                if pref[1] != COMBO:
-                    pref[2] = value
-                else:
+                if pref[1] == SPINBOX:
+                    try:
+                        pref[2][2] = int(value)
+                    except ValueError:
+                        pass
+                elif pref[1] == COMBO:
                     pref[2][1] = value
+                else:
+                    pref[2] = value
             win = SimpleDialog(self.curSource.name, prefs, self)
         win.setModal(True)
         self.connect(win, SIGNAL('editingFinished'), self._applyPrefs)
@@ -687,6 +766,8 @@ class MainWin(QWidget):
             for ts in self.__sources]
 
         index = self.sourcelist.findText(source)
+        if index == -1:
+            index = 0
         self.sourcelist.setCurrentIndex(index)
         self.__fieldsEdit.setTags(self.__sourceFields[index])
         df = get('trackpattern', u'%track% - %title%')
@@ -761,13 +842,7 @@ class MainWin(QWidget):
                 if text:
                     return self.curSource.keyword_search(text), None
                 else:
-                    if self.curSource.group_by:
-                        for primary in group:
-                            ret.extend(self.curSource.search(
-                                primary, group[primary]))
-                        return ret, files
-                    else:
-                        return self.curSource.search(files), files
+                    return tag_source_search(self.curSource, group, files)
             except RetrievalError, e:
                 return translate('WebDB',
                     'An error occured: %1').arg(unicode(e))
@@ -780,6 +855,32 @@ class MainWin(QWidget):
         self.connect(t, SIGNAL('threadfinished'), self.setResults)
         t.start()
 
+    def submit(self):
+        files = self._status['selectedfiles']
+        self.submitButton.setEnabled(False)
+
+        def end(text):
+            self.submitButton.setEnabled(True)
+            self.label.setText(text)
+
+        def submit():
+            try:
+                self.curSource.submit(files)
+            except SubmissionError, e:
+                traceback.print_exc()
+                return translate('WebDB',
+                    'An error occured: %1').arg(unicode(e))
+            except Exception, e:
+                traceback.print_exc()
+                return translate('WebDB',
+                    'An unhandled error occurred: %1').arg(unicode(e))
+
+            return translate("WebDB", "Submission completed.")
+
+        t = PuddleThread(submit, self)
+        self.connect(t, SIGNAL('threadfinished'), end)
+        t.start()
+
     def saveSettings(self):
         settings = PuddleConfig()
         settings.filename = os.path.join(SAVEDIR, 'tagsources.conf')
